function norm_data=data_arraynorm_nl(data,method,param)

switch method
	case 'quantile'
		ind=zeros(size(data));
		norm_data=zeros(size(data));
		for i=1:size(data,2)
			[norm_data(:,i),ind(:,i)]=sort(data(:,i));
		end;
		m=median(norm_data,2);
		norm_data=repmat(m,1,size(data,2));

		for i=1:size(data,2)
			norm_data(ind(:,i),i)=norm_data(:,i);
		end;

	case {'lowess','rlowess'}
		norm_data=data;

		iterations_count=3;
		for i=1:iterations_count
			med=median(norm_data,2);
			med_nan_index=find(isnan(med));
						
			for j=1:size(norm_data,2)
				index=[1:size(norm_data,1)];
				index(unique([med_nan_index ; find(isnan(norm_data(:,j)))]))=[];

				r=norm_data(index,j)-med(index);

				if method(1)=='r'
					s=lowess(med(index),r,param,5);
				else
					s=lowess(med(index),r,param);
				end;
				norm_data(index,j)=norm_data(index,j)-s;
			end;

		end;

	case {'loess','rloess'}
		norm_data=data;
		
		iterations_count=3;
		for i=1:iterations_count
			med=nan_median(norm_data,2);
			med_nan_index=find(isnan(med));
			
			for j=1:size(norm_data,2)
				index=[1:size(norm_data,1)];
				index(unique([med_nan_index ; find(isnan(norm_data(:,j)))]))=[];
				
				r=norm_data(index,j)-med(index);

%				m=med(index)-norm_data(index,i);
%				a=0.5*(med(index)+norm_data(index,i));

				if method(1)=='r'
					s=loess(med(index),r,param,1,2);

%					s=loess(a,m,param,1,2);
%					s=loess(med(index),norm_data(index,i),param,1,2);
				else
					s=loess(med(index),r,param,0,2);

%					s=loess(a,m,param,0,2);
%					s=loess(med(index),norm_data(index,i),param,0,2);
				end;

%				s=smooth(med(index),r,param,method);
%				s=smooth(med(index),norm_data(index,i),param,method);
%				s=smooth(med(index),norm_data(index,i),param,method);

				norm_data(index,j)=norm_data(index,j)-s;
				
%				norm_data(index,i)=a-0.5*(m-2*s);
%				norm_data(index,i)=norm_data(index,i)-(s-med(index));
			end;

		end;

	case {'invset_loess','rinvset_loess'}
		norm_data=data;

		iterations_count=3;
		for i=1:iterations_count
			invset=select_invariant(norm_data,floor(size(norm_data,1)/10));
			
			med=median(norm_data,2);
			med_nan_index=find(isnan(med));

			for j=1:size(norm_data,2)
				index=[1:size(norm_data,1)];
				index(unique([med_nan_index ; find(isnan(norm_data(:,j)))]))=[];

				invset_index=intersect(invset,index);
				
				r=norm_data(index,j)-med(index);
				r_full=zeros(size(med));
				r_full(index)=r;
				
				nodes_x=med(invset_index);
				nodes_y=r_full(invset_index);

				if method(1)=='r'
					s=loess(med(index),r,param,1,2,nodes_x,nodes_y);
				else
					s=loess(med(index),r,param,0,2,nodes_x,nodes_y);
				end;

				norm_data(index,j)=norm_data(index,j)-s;
			end;
		end;

	case {'cyclic_loess','rcyclic_loess'}
		norm_data=data;
		correction=zeros(size(data));

		iterations_count=3;
		for i=1:iterations_count
			for j=1:size(norm_data,2)
				for k=j+1:size(norm_data,2)
					
					index=[1:size(norm_data,1)];
					index(unique([find(isnan(norm_data(:,j))) ; find(isnan(norm_data(:,k)))]))=[];

					m=norm_data(index,j)-norm_data(index,k);
					a=0.5*(norm_data(index,j)+norm_data(index,k));
					if method(1)=='r'
						s=loess(a,m,param,1,2);
					else
						s=loess(a,m,param,0,2);
					end;
					ms=0.5*(m-s);
					correction(index,j)=correction(index,j)+(a+ms-norm_data(index,j));
					correction(index,k)=correction(index,k)+(a-ms-norm_data(index,k));
				end;
			end;
			correction=correction/(size(norm_data,2)-1);
			max(abs(correction(:)))
			norm_data(index,:)=norm_data(index,:)+correction(index,:);
		end;
end;
return;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function invset=select_invariant(data,ranges_count)

nan_index=sum(isnan(data),2);
data(find(nan_index==size(data,2)),:)=[];

range_points_count=floor(length(data)/ranges_count);

m=nanmean(data,2);
[m,index]=sort(m);

invset=zeros(ranges_count,1);
for i=1:ranges_count
	start=(i-1)*range_points_count+1;
	stop=start+range_points_count-1;

	range_data=data(index(start:stop),:);
	cv=sqrt(nan_sd(range_data')')./nan_mean(range_data,2);

	min_index=find(cv==min(cv));
	invset(i)=index(start+min_index(1)-1);
end;
