function [data,list,n]=data_rollup(data,order,config,gcnames)

if nargin<2
	error('At least 2 arguments required');
elseif nargin==2
	config=struct('method','median','min_features',1,'min_nonmissing',0.0,'dim',1,'log_base',0);
elseif nargin==3 && ~isstruct(config)
	error('config must be a structure');
end;

n=ones(size(data,config.dim),1);

groups=sort(unique(order));

waitbar_hnd=waitbar(0,'Urednianie...');

for i=1:length(groups)
	index=find(order==groups(i));
	if ~isempty(index)
		%if (length(index)>=config.min_features)
			if length(index)>1
				if config.dim==1
					
					[data(index(1),:),scaled_data,ref_index,outliers,er]=rollup_features(data(index,:),config);
					n(index(1))=length(index);
					if er==1
						%disp(sprintf('protein=%s',gcnames{i}));
					end;
					
					data(index(2:end),:)=-Inf;
				else
					data(:,index(1))=rollup_samples(data(:,index),config);
					data(:,index(2:end))=-Inf;
				end;
			end;
		%else
		%	if config.dim==1
		%		data(index,:)=-Inf;
		%	else
		%		data(:,index)=-Inf;
		%	end;
		%end;
	end;
	waitbar(i/length(groups),waitbar_hnd);
end;

if config.dim==1
	list=find(sum(isinf(data),2)==size(data,2));
	data(list,:)=[];
	n(list)=[];
else
	list=find(sum(isinf(data),1)==size(data,1));
	data(:,list)=[];
	n(list)=[];
end;

delete(waitbar_hnd);
return;

%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function rdata=rollup_samples(data,config)

switch config.method
	case 'mean'
		rdata=nan_mean(data,2);
	otherwise
		rdata=nan_median(data,2);	
end;
if size(data,2)==3
	idx=(sum(~isnan(data),2)<1);
else
	idx=(sum(~isnan(data),2)<2);
end	
rdata(idx)=NaN;
return;

%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [r,scaled_data,ref_index,outliers,er]=rollup_features(data,config)

if nargin<1
	error('At least one argument required');
elseif nargin==1
	config=struct('method','median','min_features',1,'min_nonmissing',0.0,'dim',1,'log_base',0);
elseif nargin==2 && ~isstruct(config)
	error('config must be a structure');
end;

r=NaN*ones(1,size(data,2));
scaled_data=[];
ref_index=0;
outliers=[];
er=0;

switch config.method
	case 'mean'
		for i=1:size(data,2)
			if config.min_nonmissing==0 || check_nan(data(:,i),config.min_nonmissing)==1
				r(i)=nan_mean(data(:,i));
			end;
		end;

	case 'gmean'
		for i=1:size(data,2)
			if config.min_nonmissing==0 || check_nan(data(:,i),config.min_nonmissing)==1
				r(i)=nan_gmean(data(:,i));
			end;
		end;
		
	case 'median'
		for i=1:size(data,2)
			if config.min_nonmissing==0 || check_nan(data(:,i),config.min_nonmissing)==1
				r(i)=nan_median(data(:,i));
			end;
		end;
		
	case 'max'
		m=nan_median(data');
		ref_index=find(m==max(m));
		if ~isempty(ref_index)
			r=data(ref_index(1),:);
		end;
		
	case 'r-rollup'
		%data
		ref_index=reference_index(data);
		if (ref_index>0)
			if (config.log_base==0)
				scaling_ratios=data./repmat(data(ref_index,:),size(data,1),1);
				scaling_ratios_overlap=sum(~isnan(scaling_ratios),2);
				
				scaling_ratios_medians=nan_median(scaling_ratios,2);
				scaling_ratios_medians(find(scaling_ratios_overlap<config.min_overlap))=1;		%unikniecie sytuacji, w ktorej czynnik skalujacy dla wiersza jest NaN
				
				scaled_data=data./repmat(scaling_ratios_medians,1,size(data,2));
			else
				scaling_ratios=data-repmat(data(ref_index,:),size(data,1),1);
				scaling_ratios_overlap=sum(~isnan(scaling_ratios),2);
				
				scaling_ratios_medians=nan_median(scaling_ratios,2);
				scaling_ratios_medians(find(scaling_ratios_overlap<config.min_overlap))=0;		%unikniecie sytuacji, w ktorej czynnik skalujacy dla wiersza jest NaN
				
				scaled_data=data-repmat(scaling_ratios_medians,1,size(data,2));
			end;
			
			if size(scaled_data,1)>=config.grubbs_min_values
				for i=1:size(scaled_data,2)
					if (sum(~isnan(scaled_data(:,i)))>=config.grubbs_min_values)
						h=-1;
						while h~=0
							[h,pval,g,idx]=grubbs(scaled_data(:,i),config.grubbs_p_th);
							%h
							if isnan(pval)
								%disp(sprintf('sample=%d',i));
								er=1;
								%scaled_data(:,i)
							end
							if h~=0
								%disp(sprintf('%d %f',i,pval));
								scaled_data(idx,i)=NaN;
								scaled_data(idx,i)=nan_median(scaled_data(:,i));
							end;
						end;
					end;
				end;
			end;	
			
			for i=1:size(data,2)
				if config.min_nonmissing==0 || check_nan(scaled_data(:,i),config.min_nonmissing)==1
					r(i)=nan_median(scaled_data(:,i));
				end;
			end;
		end;
end;
return;

%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [ref_index]=reference_index(data)

%TODO rozpatrzyc przypadek bez zadnych nieNaN
m=nan_median(data,2);
if sum(isnan(m))==size(data,1)
	ref_index=0;
else
	nonmissing_count=sum(~isnan(data),2);

	max_nonmissing_indices=find(nonmissing_count==max(nonmissing_count));
	max_median_indices=find(m(max_nonmissing_indices)==max(m(max_nonmissing_indices)));

	ref_index=max_nonmissing_indices(max_median_indices(1));
end;

return;
%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function result=check_nan(data,min_nonmissing)

result=0;

if sum(~isnan(data))/length(data)>=min_nonmissing
	result=1;
end;

return;