function clustering

Array_id=findobj(0,'Tag','ArrayView');
config=get(findobj(Array_id,'Tag','FileOpen'),'UserData');

SAMPLES=1;
FEATURES=2;

size_x=495;
size_y=240;

method_string={'hierarchical';'K-means';'LVQ';'DBSCAN';'OPTICS';'NBC';'Jarvis-Patrick'};
dist_string={'euclidean';'seuclidean';'correlation';'cosine';'spearman';'mahalanobis';'cityblock';'chebychev'};
kmeans_centroid_string={'mean','median'};
hier_linkage_string={'single';'complete';'average';'weighted';'ward';'centroid';'median'};
hier_clusters_string={'distance thr.';'fixed number'};
normalize_string={'standarize';'min-max'};

data=get(Array_id,'UserData');

if strcmp(get(gcbo,'Tag'),'ClusterSamples')
    ctype=SAMPLES;    
    name='Klasteryzacja prbek';
    tag='ACluster';
    
    order=get(findobj(Array_id,'Tag','ArrayClassAxes'),'UserData');    
    cnames=get(findobj(Array_id,'Tag','GeneClassPop'),'UserData');    
else
    ctype=FEATURES;    
    name='Klasteryzacja cech';
    tag='GCluster';
        
    data=data';
    order=get(findobj(Array_id,'Tag','GeneClassAxes'),'UserData');    
    cnames=get(findobj(Array_id,'Tag','ArrayClassPop'),'UserData');        
end

features_count=size(data,1);
vectors_count=size(data,2);
classes_count=length(unique(order));
groups_count=size(cnames,1);    
    
if groups_count>1
    groups_names=cell(groups_count+1,1);
    for i=2:groups_count+1
        groups_names{i}=cnames{i-1};
    end
else
    groups_names=cell(groups_count,1);
end
groups_names{1}='full dataset';

    
ACluster_id=findobj('Tag',tag);
if isempty(ACluster_id)==0
	figure(ACluster_id);
else
	ACluster_id=figure('Units','pixels', ...
		'Position',[figpos(Array_id,size_x,size_y) size_x size_y],...
		'Color',config.bg_color, ...
		'Visible','on',...
		'MenuBar','none', ...
		'NumberTitle','off', ...
		'Resize','off',...
		'CreateFcn','set(findobj(0,''Tag'',''ArrayView''),''WindowButtonMotionFcn'','''');',...
		'DeleteFcn','set(findobj(0,''Tag'',''ArrayView''),''WindowButtonMotionFcn'',''mstat_mf'');',...
		'Name',name, ...
		'KeyPressFcn',@figure_kb_cb,...				
		'Tag',tag);

    %Algorithm
	%%%%%%%%%%
	hnd=uipanel('Parent',ACluster_id, ...
		'Title','Klasteryzacja:', ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 170 240 65], ...
		'Tag','AlgorithmPanel');
    
	uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 28 120 18], ...
		'HorizontalAlignment','left', ...
		'String','Algorytm: ', ...
		'Style','text', ...
		'Tag','MethodText');
    
	uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',[1 1 1], ...
		'Position',[130 30 100 20], ...
		'String',method_string, ...
		'Style','popupmenu',...
		'Value',1, ...
		'Tag','MethodPop', ...
		'Callback','clustering_cb(''method_pop'')');

    uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 3 120 18], ...
		'HorizontalAlignment','left', ...
		'String','Miara odlegoci: ', ...
		'Style','text', ...
		'Tag','DistText');
    
    val=1;
    if ctype==FEATURES
        val=3;
    end;

	uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',[1 1 1], ...
		'Position',[130 5 100 20], ...
		'String',dist_string, ...
		'Style','popupmenu',...
		'Value',val, ...
		'Tag','DistPop', ...
		'Callback','clustering_cb(''distance_pop'')');

    %Data
	%%%%%%%%%%
	hnd=uipanel('Parent',ACluster_id, ...
		'Title','Dane:', ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 125 240 45], ...
		'Tag','DataPanel');
    
    uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 3 120 18], ...
		'HorizontalAlignment','left', ...
		'String','Na podstawie: ', ...
		'Style','text', ...
		'Tag','GroupText');
    
	uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',[1 1 1], ...
		'Position',[130 7 100 20], ...
		'String',groups_names, ...
		'Style','popupmenu',...
		'Value',1, ...
		'Tag','GroupPop', ...
		'Callback','clustering_cb(''group_pop'')');
    
    %Algorithm:kmeans
	%%%%%%%%%%
	hnd=uipanel('Parent',ACluster_id, ...
		'Title','Parametry:', ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 35 240 90], ...
        'Visible','off',...
		'Tag','KmeansPanel');

    uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 53 120 18], ...
		'HorizontalAlignment','left', ...
		'String','Centroid: ', ...
		'Style','text', ...
		'Tag','KmeansCentroidPopText');
    
    uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',[1 1 1], ...
		'Position',[130 55 100 20], ...
		'String',kmeans_centroid_string, ...
		'Style','popupmenu',...
		'Value',1, ...
		'Tag','KmeansCentroidPop', ...
		'Callback','');
    
    uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 28 120 18], ...
		'HorizontalAlignment','left', ...
		'String','Liczba klastrw:', ...
		'Style','text', ...
		'Tag','KmeansClustersEditText');
    
	uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',[1 1 1], ...
		'Position',[180 30 50 20], ...
		'HorizontalAlignment','right', ...
		'String',num2str(max(classes_count,2)),...
		'Style','edit', ...
		'Tag','KmeansClustersEdit',...
		'UserData',[1 vectors_count num2str(max(classes_count,2)) 1],...
		'Callback','edit_cb');

    uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 3 120 18], ...
		'HorizontalAlignment','left', ...
		'String','Liczba iteracji: ', ...
		'Style','text', ...
		'Tag','KmeansIterationsEditText');
    
	uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',[1 1 1], ...
		'Position',[180 5 50 20], ...
		'HorizontalAlignment','right', ...
		'String','10',...
		'Style','edit', ...
		'Tag','KmeansIterationsEdit',...
		'UserData',[1 inf 10 0],...
		'Callback','edit_cb');

    %Algorithm:lvq
	%%%%%%%%%%
	hnd=uipanel('Parent',ACluster_id, ...
		'Title','Parametry:', ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 35 240 90], ...
        'Visible','off',...
		'Tag','LvqPanel');
    
    uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 53 120 18], ...
		'HorizontalAlignment','left', ...
		'String','Liczba klastrw:', ...
		'Style','text', ...
		'Tag','LvqClustersEditText');
    
	uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',[1 1 1], ...
		'Position',[180 55 50 20], ...
		'HorizontalAlignment','right', ...
		'String',num2str(max(classes_count,2)),...
		'Style','edit', ...
		'Tag','LvqClustersEdit',...
		'UserData',[1 vectors_count num2str(max(classes_count,2)) 1],...
		'Callback','edit_cb');

    uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 28 120 18], ...
		'HorizontalAlignment','left', ...
		'String','Liczba epok/iteracji: ', ...
		'Style','text', ...
		'Tag','LvqIterationsEditText');

    uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',[1 1 1], ...
		'Position',[130 30 50 20], ...
		'HorizontalAlignment','right', ...
		'String','10',...
		'Style','edit', ...
		'UserData',[1 inf 10 0],...        
		'Tag','LvqEpochsEdit',...
		'Callback','edit_cb');
    
	uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',[1 1 1], ...
		'Position',[180 30 50 20], ...        
		'HorizontalAlignment','right', ...
		'String','10',...
		'Style','edit', ...
		'UserData',[1 inf 10 0],...        
		'Tag','LvqIterationsEdit',...
		'Callback','edit_cb');
    
    uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 3 120 18], ...
		'HorizontalAlignment','left', ...
		'String','Staa uczca: ', ...
		'Style','text', ...
		'Tag','LvqEtaEditText');
    
	uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',[1 1 1], ...
		'Position',[180 5 50 20], ...
		'HorizontalAlignment','right', ...
		'String','0.1',...
		'Style','edit', ...
		'Tag','LvqEtaEdit',...
		'UserData',[0 1 0.1 1],...
		'Callback','edit_cb');
    
    %Algorithm:hierarchical
	%%%%%%%%%%
	hnd=uipanel('Parent',ACluster_id, ...
		'Title','Parametry:', ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 35 240 90], ...
        'Visible','on',...        
		'Tag','HierPanel');

    uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 53 120 18], ...
		'HorizontalAlignment','left', ...
		'String','Sposb czenia: ', ...
		'Style','text', ...
		'Tag','HierLinkagePopText');
    
	uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',[1 1 1], ...
		'Position',[130 55 100 20], ...
		'String',hier_linkage_string, ...
		'Style','popupmenu',...
		'Value',3, ...
		'Tag','HierLinkagePop', ...
		'Callback','clustering_cb(''linkage_pop'')');

	uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 30 120 18], ...
		'String','Utwrz klastry:',...
		'Style','checkbox', ...
		'Value',0,...
		'Tag','HierClustersCheck',...
		'Callback','clustering_cb(''create_clusters_check'')');

	uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',[1 1 1], ...
		'Position',[130 30 100 20], ...
		'String',hier_clusters_string, ...
		'Style','popupmenu',...
		'Enable','off',...
		'Value',2, ...
		'Tag','HierClustersPop', ...
		'Callback','clustering_cb(''create_clusters_pop'')');

	uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',[1 1 1], ...
		'Position',[180 5 50 20], ...
		'HorizontalAlignment','right', ...
		'String',num2str(max(classes_count,2)),...
		'Style','edit', ...
		'Enable','off',...        
		'Tag','HierClustersEdit',...
		'UserData',[1 vectors_count num2str(max(classes_count,2)) 1],...
		'Callback','edit_cb');

    %Algorithm:DBSCAN
	%%%%%%%%%%
	hnd=uipanel('Parent',ACluster_id, ...
		'Title','Parametry:', ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 35 240 90], ...
        'Visible','off',...
		'Tag','DbscanPanel');
    
    uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 53 120 18], ...
		'HorizontalAlignment','left', ...
		'String','MinPts:', ...
		'Style','text', ...
		'Tag','DbscanMinPtsEditText');
    
    val=5;
	uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',[1 1 1], ...
		'Position',[180 55 50 20], ...
		'HorizontalAlignment','right', ...
		'String',5,...
		'Style','edit', ...
		'Tag','DbscanMinPtsEdit',...
		'UserData',[0 inf val 1],...
		'Callback','edit_cb');

    uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 28 120 18], ...
		'HorizontalAlignment','left', ...
		'String','Eta:', ...
		'Style','text', ...
		'Tag','DbscanEtaEditText');
    
    val=1;
	uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',[1 1 1], ...
		'Position',[180 30 50 20], ...
		'HorizontalAlignment','right', ...
		'String',1,...
		'Style','edit', ...
		'Tag','DbscanEtaEdit',...
		'UserData',[0 inf val 1],...
		'Callback','edit_cb');

    uicontrol('Parent',hnd, ...
 		'Units','pixels', ...
 		'Position',[130 5 100 20], ...
 		'String','DBSCAN plot', ...
 		'Tag','DbscanPlotButt', ...
 		'CallBack','clustering_cb(''dbscan_plot'')');

    %Algorithm:OPTICS
	%%%%%%%%%%
	hnd=uipanel('Parent',ACluster_id, ...
		'Title','Parametry:', ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 35 240 90], ...
        'Visible','off',...
		'Tag','OpticsPanel');
    
    uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 53 120 18], ...
		'HorizontalAlignment','left', ...
		'String','MinPts:', ...
		'Style','text', ...
		'Tag','OpticsMinPtsEditText');
    
    val=5;
	uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',[1 1 1], ...
		'Position',[180 55 50 20], ...
		'HorizontalAlignment','right', ...
		'String',5,...
		'Style','edit', ...
		'Tag','OpticsMinPtsEdit',...
		'UserData',[0 inf val 1],...
		'Callback','edit_cb');

    uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 28 120 18], ...
		'HorizontalAlignment','left', ...
		'String','Eta:', ...
		'Style','text', ...
		'Tag','OpticsEtaEditText');

    val=1;
	uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',[1 1 1], ...
		'Position',[180 30 50 20], ...
		'HorizontalAlignment','right', ...
		'String',1,...
		'Style','edit', ...
		'Tag','OpticsEtaEdit',...
		'UserData',[0 inf val 1],...
		'Callback','edit_cb');

    uicontrol('Parent',hnd, ...
 		'Units','pixels', ...
 		'Position',[130 5 100 20], ...
 		'String','OPTICS plot', ...
 		'Tag','OpticsPlotButt', ...
 		'CallBack','clustering_cb(''optics_plot'')');

    %Algorithm:J-P
	%%%%%%%%%%
	hnd=uipanel('Parent',ACluster_id, ...
		'Title','Parametry:', ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 35 240 90], ...
        'Visible','off',...
		'Tag','JpPanel');
    
    uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 53 120 18], ...
		'HorizontalAlignment','left', ...
		'String','Rozmiar ssiedztwa:', ...
		'Style','text', ...
		'Tag','JpKEditText');
    
    val=5;
	uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',[1 1 1], ...
		'Position',[180 55 50 20], ...
		'HorizontalAlignment','right', ...
		'String',5,...
		'Style','edit', ...
		'Tag','JpKEdit',...
		'UserData',[2 vectors_count val 0],...
		'Callback','edit_cb');

    uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 28 160 18], ...
		'HorizontalAlignment','left', ...
		'String','Liczba wsplnych ssiadw:', ...
		'Style','text', ...
		'Tag','JpLEditText');
    
    val=3;
	uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',[1 1 1], ...
		'Position',[180 30 50 20], ...
		'HorizontalAlignment','right', ...
		'String',val,...
		'Style','edit', ...
		'Tag','JpLEdit',...
		'UserData',[2 vectors_count val 0],...
		'Callback','edit_cb');

    %Algorithm:NBC
	%%%%%%%%%%
	hnd=uipanel('Parent',ACluster_id, ...
		'Title','Parametry:', ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 35 240 90], ...
        'Visible','off',...
		'Tag','NbcPanel');
    
    uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 53 120 18], ...
		'HorizontalAlignment','left', ...
		'String','Rozmiar ssiedztwa:', ...
		'Style','text', ...
		'Tag','NbcKEditText');
    
    val=5;
	uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',[1 1 1], ...
		'Position',[180 55 50 20], ...
		'HorizontalAlignment','right', ...
		'String',5,...
		'Style','edit', ...
		'Tag','NbcKEdit',...
		'UserData',[1 vectors_count val 0],...
		'Callback','edit_cb');
    
    %Preprocessing
	%%%%%%%%%%
	hnd=uipanel('Parent',ACluster_id, ...
		'Title','Preprocessing:', ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[250 170 240 65], ...
		'Tag','PreprocessingPanel');

    val=1;
    if ctype==FEATURES
        val=0;
    end;
    
    uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 30 230 18], ...
		'String','Normalizacja cech:',...
		'Style','checkbox', ...
		'Value',val,...
		'Tag','NormalizeCheck',...
		'Callback','');

    uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',[1 1 1], ...
		'Position',[130 30 100 20], ...
		'String',normalize_string, ...
		'Style','popupmenu',...
		'Value',1, ...
        'Enable','off',...
		'Tag','NormalizePop', ...
		'Callback','');
    
	uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 5 150 18], ...
		'String','Skadowe gwne:',...
		'Style','checkbox', ...
		'Value',0,...
		'Tag','PcaCheck',...
		'Callback','clustering_cb(''pca_check'')');

	uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',[1 1 1], ...
		'Position',[180 5 50 20], ...
		'HorizontalAlignment','right', ...
		'Enable','off',...
		'String',num2str(min(features_count,vectors_count-1)),...
		'UserData',min(features_count,vectors_count-1),...
		'Style','edit', ...
		'Tag','PcaEdit',...
		'UserData',[1 min(features_count,vectors_count-1) min(features_count,vectors_count-1) 0],...
		'Callback','edit_cb');


    %Postprocessing
	%%%%%%%%%%
	hnd=uipanel('Parent',ACluster_id, ...
		'Title','Postprocessing:', ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[250 125 240 45], ...
		'Tag','PostprocessingPanel');
    
	uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 5 180 18], ...
		'String','Uprzdkuj prbki po klasteryzacji',...
		'Style','checkbox', ...
		'Value',0,...
		'Enable','off',...
		'Tag','ReorderCheck',...
		'Callback','');

    %Quality
	%%%%%%%%%%
	hnd=uipanel('Parent',ACluster_id, ...
		'Title','Miary jakoci:', ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[250 35 240 90], ...
        'Visible','on',...        
		'Tag','QualityPanel');
    
    uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',config.bg_color, ...
		'Position',[5 53 50 18], ...
		'HorizontalAlignment','left', ...
		'String','Sil: ', ...
		'Style','text', ...
		'Tag','QualitySilEditText');
       
	uicontrol('Parent',hnd, ...
		'Units','pixels', ...
		'BackgroundColor',[1 1 1], ...
		'Position',[50 55 60 20], ...
		'HorizontalAlignment','right', ...
		'String',0,...
		'Style','edit', ...
		'Tag','QualitySilEdit',...
 		'Enable','on',...                
		'Callback','edit_cb');
    
 	uicontrol('Parent',hnd, ...
 		'Units','pixels', ...
 		'Position',[130 55 100 20], ...
 		'String','Silhouette', ...
 		'Enable','on',...
 		'Tag','QualitySilButt', ...
 		'CallBack','clustering_cb(''sil_plot'')');

    %Control
    %%%%%%%%%%%%
	uicontrol('Parent',ACluster_id, ...
		'Units','pixels', ...
		'Position',[410 5 80 25], ...
		'String','Wykonaj', ...
		'Tag','ClusterButt', ...
		'CallBack','clustering_cb(''cluster'')');

	uicontrol('Parent',ACluster_id, ...
		'Units','pixels', ...
		'Position',[320 5 80 25], ...
		'String','Zamknij', ...
		'Tag','CancelButt', ...
		'Callback','delete(gcbf);');
end;