#include <stdio.h>
#include <string.h>
#include "mex.h"

#define MAXLINE 65536

unsigned int getLinesCount(FILE *file)
{
	char			*line;
	unsigned int	lineCounter=0;

	if (file!=NULL)
	{
		line=mxCalloc(MAXLINE+1,sizeof(char));

		while (fgets(line,MAXLINE,file)!=NULL)
			if (strtok(line,"\n\r")!=NULL)
				lineCounter++;

		mxFree(line);
	}

	rewind(file);
	return(lineCounter);
}

unsigned int getTokensCount(FILE *file)
{
	char			*line;
	unsigned int	tokenCounter=0;

	if (file!=NULL)
	{
		line=mxCalloc(MAXLINE+1,sizeof(char));		

		while (fgets(line,MAXLINE,file)!=NULL)
			if (strtok(line,"\n\r")!=NULL)
			{
				strtok(line,"\t\n\r");
				tokenCounter=1;
				while (strtok(NULL,"\t\n\r")!=NULL)
					tokenCounter++;
				break;
			}

		mxFree(line);
	}
	rewind(file);
	return(tokenCounter);
}

void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
{
	mxArray			*cellMtx;
	FILE			*file;
	char			*fileName;
	char			*line,*name;
	unsigned int	strLength,linesCount,tokensCount;
	unsigned int	i=0;
	

	if (nrhs!=1)
		mexErrMsgTxt("Two input arguments required.");
	if (nlhs>2)
		mexErrMsgTxt("Too many output arguments");
	if (mxIsChar(prhs[0])!=1)
		mexErrMsgTxt("Input must be a string.");
	if (mxGetM(prhs[0]) != 1)
		mexErrMsgTxt("Input must be a row vector.");

	strLength=(mxGetM(prhs[0])*mxGetN(prhs[0]))+1;
	fileName=mxCalloc(strLength,sizeof(char));

	if (mxGetString(prhs[0],fileName,strLength)!=0) 
		mexWarnMsgTxt("Not enough space. String is truncated.");
	
	if ((file=fopen(fileName,"rt"))!=NULL)
	{
		line=mxCalloc(MAXLINE+1,sizeof(char));
		
		linesCount=getLinesCount(file);
		if (linesCount==1)
		{
			tokensCount=getTokensCount(file);
			cellMtx=mxCreateCellMatrix(tokensCount,1);

			while (fgets(line,MAXLINE,file)!=NULL)
				if (strtok(line,"\n\r")!=NULL)
				{
					name=strtok(line,"\t\n\r");
//					mxSetCell(cellMtx,i++,mxCreateString(line));
					mxSetCell(cellMtx,i++,mxCreateString(name));
					while ((name=strtok(NULL,"\t\n\r"))!=NULL && i<tokensCount)
						mxSetCell(cellMtx,i++,mxCreateString(name));
					break;
				}
		}
		else
		{
			cellMtx=mxCreateCellMatrix(linesCount,1);

			while (fgets(line,MAXLINE,file)!=NULL && i<linesCount)
				if (strtok(line,"\n\r")!=NULL)
					mxSetCell(cellMtx,i++,mxCreateString(line));
		}
		fclose(file);
		mxFree(line);
	}
	else
		cellMtx=mxCreateCellMatrix(0,0);

	mxFree(fileName);

	plhs[0]=cellMtx;
	if (nlhs>1)
		plhs[1]=mxCreateDoubleScalar((double)i);
}
