#include <stdio.h>
#include <string.h>
#include "mex.h"

#define MAXLINE 65536

unsigned int getLinesCount(FILE *file)
{
	char			*line;
	unsigned int	lineCounter=0;

	if (file!=NULL)
	{
		line=mxCalloc(MAXLINE+1,sizeof(char));

		while (fgets(line,MAXLINE,file)!=NULL)
			if (strtok(line,"\n\r")!=NULL)
				lineCounter++;

		mxFree(line);
	}

	rewind(file);
	return(lineCounter);
}

char isEmpty(char *line)
{
	char			isempty=1;
	unsigned int	i,lineLength;
	
	lineLength=strlen(line);
	for (i=0;i<lineLength;i++)
		if (line[i]!=' ' && line[i]!='\t' && line[i]!='\n' && line[i]!='\r')
		{
			isempty=0;
			break;
		}
	return(isempty);
}

unsigned char isWhiteChar(char chr)
{
	unsigned char	isWhite=0;
	
//	if (chr==' ' || chr=='\t' || chr=='\n' || chr=='\r')
	if (chr=='\t' || chr=='\n' || chr=='\r')
		isWhite=1;

	return(isWhite);
}

char* trim(char *str)
{
	while (isWhiteChar(*str))
//	while (*str==' ' || *str=='\t' || *str=='\n' || *str=='\r');
		str++;

	return(str);
}

unsigned int getTokensCount(char *line)
{
	char			*str;
	unsigned int	i,lineLength;
	unsigned int	tokenCounter=0;

	if (line!=NULL && (lineLength=strlen(line))>0)
	{
		str=trim(line);
		
		while (*str)
		{
			tokenCounter++;
			while (*str && !isWhiteChar(*str))
				str++;
			str=trim(str);
		}
	}
	return(tokenCounter);
}

void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
{
	mxArray			*cellMtx,*lineMtx;
	FILE			*file;
	char			*fileName,*line,*name;
	unsigned int	strLength,linesCount,tokensCount;
	unsigned int	i,lineCounter;
	

	if (nrhs!=1)
		mexErrMsgTxt("Two input arguments required.");
	if (nlhs>2)
		mexErrMsgTxt("Too many output arguments");
	if (mxIsChar(prhs[0])!=1)
		mexErrMsgTxt("Input must be a string.");
	if (mxGetM(prhs[0]) != 1)
		mexErrMsgTxt("Input must be a row vector.");

	strLength=(mxGetM(prhs[0])*mxGetN(prhs[0]))+1;
	fileName=mxCalloc(strLength,sizeof(char));

	if (mxGetString(prhs[0],fileName,strLength)!=0) 
		mexWarnMsgTxt("Not enough space. String is truncated.");
	
	if ((file=fopen(fileName,"rt"))!=NULL)
	{
		linesCount=getLinesCount(file);
		if (linesCount>0)
		{
			line=mxCalloc(MAXLINE+1,sizeof(char));
			cellMtx=mxCreateCellMatrix(linesCount,1);

			lineCounter=0;
			while (fgets(line,MAXLINE,file)!=NULL)
				if ((tokensCount=getTokensCount(line))>0)
				{
					lineMtx=mxCreateCellMatrix(1,tokensCount);
					name=strtok(line,"\t\n\r");
					
					i=0;
					mxSetCell(lineMtx,i++,mxCreateString(name));

					while ((name=strtok(NULL,"\t\n\r"))!=NULL)
						mxSetCell(lineMtx,i++,mxCreateString(name));

					mxSetCell(cellMtx,lineCounter++,lineMtx);
				}
			mxFree(line);
		}
		else
			cellMtx=mxCreateCellMatrix(0,0);
		fclose(file);
	}
	else
		cellMtx=mxCreateCellMatrix(0,0);

	mxFree(fileName);
	
	plhs[0]=cellMtx;
}
