#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "mex.h"

#define	DEF_FORMAT	"%0.2f"
#define	DEF_DELIM	'\t'
#define MAXLINE		65536

void writeMatrix(const mxArray *mtx,char *fileName,char *format)
{
	FILE			*file=NULL;
	double			*mtxData=NULL,value=-1.0;
	unsigned int	xSize=0,ySize=0;
	int				i=0,j=0;
	
	if 	(mtx!=NULL)
	{
		xSize=mxGetN(mtx);
		ySize=mxGetM(mtx);
		mtxData=mxGetPr(mtx);

		if (xSize!=0 && ySize!=0 && (file=fopen(fileName,"wt"))!=NULL)
		{
//			xSize-=1;
			for (i=0;i<ySize;i++)
			{
				for (j=0;j<xSize;j++)
				{
					value=*(mtxData+ySize*j+i);
					if (mxIsNaN(value))
						fprintf(file,"NaN");
					else if (mxIsInf(value))
					{
						if (value>0)
							fprintf(file,"Inf");
						else
							fprintf(file,"-Inf");
					}
					else
						fprintf(file,format,value);
					if (j<xSize-1)
						fprintf(file,"%c",DEF_DELIM);
					else
						fprintf(file,"\n");
				}
/*				value=*(mtxData+ySize*j+i);
				if (mxIsNaN(value))
					fprintf(file,"NaN");
				else if (mxIsInf(value))
				{
					if (value>0)
						fprintf(file,"Inf");
					else
						fprintf(file,"-Inf");
				}
				else
					fprintf(file,format,value);
				fprintf(file,"\n");*/
			}
			fclose(file);
		}
	}
}
/*
*/
int writeTable(const mxArray *structure,char *fileName,char *format)
{
	int				result=0;
	FILE			*file=NULL;
	mxArray			*name=NULL,*mtx=NULL,*columnNames=NULL,*rowNames=NULL,*columnClasses=NULL,*classNames=NULL,*cell=NULL;
	double			*mtxData=NULL,*orderData=NULL;
	double			value;
	char			*cellString=NULL;
	unsigned int	cellStringLength=0;
	unsigned int	xSize=0,ySize=0;
	unsigned int	i=0,j=0;
	
	if 	(structure!=NULL)
	{
		mtx=mxGetField(structure,0,"data");
		name=mxGetField(structure,0,"name");
		columnNames=mxGetField(structure,0,"column_names");
		rowNames=mxGetField(structure,0,"row_names");
		columnClasses=mxGetField(structure,0,"column_classes");
		classNames=mxGetField(structure,0,"class_names");

		if (mtx!=NULL && name!=NULL && columnNames!=NULL && rowNames!=NULL && columnClasses!=NULL && classNames!=NULL)
		{
			xSize=mxGetN(mtx);
			ySize=mxGetM(mtx);

			if (xSize!=0 && ySize!=0 && (file=fopen(fileName,"wt"))!=NULL)
			{
				cellString=mxCalloc(MAXLINE+1,sizeof(char));

				//zapisanie nazwy zbioru
				fputs("#",file);
				cellStringLength=mxGetM(name)*mxGetN(name)+1;
				mxGetString(name,cellString,(cellStringLength>MAXLINE)?MAXLINE+1:cellStringLength);
				fputs(cellString,file);

				//zapisanie nazw kolumn
				for (i=0;i<xSize;i++)
				{
					fprintf(file,"%c",DEF_DELIM);
					cell=mxGetCell(columnNames,i);
					cellStringLength=mxGetM(cell)*mxGetN(cell)+1;
					mxGetString(cell,cellString,(cellStringLength>MAXLINE)?MAXLINE+1:cellStringLength);
					fputs(cellString,file);
				}
				fprintf(file,"\n");

				//zapisanie nazw klas i podzialu na klasy
				for (i=0;i<mxGetM(classNames);i++)
				{
					fputs("#",file);
					cell=mxGetCell(classNames,i);
					cellStringLength=mxGetM(cell)*mxGetN(cell)+1;
					mxGetString(cell,cellString,(cellStringLength>MAXLINE)?MAXLINE+1:cellStringLength);
					fputs(cellString,file);
				}

				orderData=mxGetPr(columnClasses);
				for (i=0;i<xSize;i++)
				{
					fprintf(file,"%c",DEF_DELIM);
					fprintf(file,"%d",(int)(*(orderData+i)));
				}
				fprintf(file,"\n");

				//zapisanie nazw wierszy i tablicy
				mtxData=mxGetPr(mtx);
				for (i=0;i<ySize;i++)
				{
					cell=mxGetCell(rowNames,i);
					cellStringLength=mxGetM(cell)*mxGetN(cell)+1;
					mxGetString(cell,cellString,(cellStringLength>MAXLINE)?MAXLINE+1:cellStringLength);
					fputs(cellString,file);

					for (j=0;j<xSize;j++)
					{
						value=*(mtxData+ySize*j+i);
						fprintf(file,"%c",DEF_DELIM);
						if (mxIsNaN(value))
							fprintf(file,"NaN");
						else if (mxIsInf(value))
						{
							if (value>0)
								fprintf(file,"Inf");
							else
								fprintf(file,"-Inf");
						}
						else
							fprintf(file,format,value);
					}
					fprintf(file,"\n");
				}

				fclose(file);
				result=1;
			}
		}
	}
	return(result);
}

void mexFunction(int nlhs,mxArray *plhs[],int nrhs,const mxArray *prhs[])
{
	mxArray			*mtx;
	char			*fileName,*format=NULL;
	unsigned int	strLength;

	if (nrhs<2)
		mexErrMsgTxt("Two input arguments required.");
	if (mxIsChar(prhs[1])!=1 && mxGetM(prhs[1])!=1)
		mexErrMsgTxt("File name must be a string.");
	if (nrhs>2 && mxIsChar(prhs[2])!=1 && mxGetM(prhs[2])!=1)
		mexErrMsgTxt("Format must be a string.");

	strLength=(mxGetM(prhs[1])*mxGetN(prhs[1]))+1;
	fileName=mxCalloc(strLength,sizeof(char));
	if (mxGetString(prhs[1],fileName,strLength)!=0) 
		mexWarnMsgTxt("File name string is truncated.");

	if (nrhs>2)
	{
		strLength=(mxGetM(prhs[2])*mxGetN(prhs[2]))+1;
		format=mxCalloc(strLength,sizeof(char));
		if (mxGetString(prhs[2],format,strLength)!=0) 
			mexWarnMsgTxt("Format string is truncated.");
	}
	else
		format=DEF_FORMAT;

	if (mxIsStruct(prhs[0]))
	{
		//mexWarnMsgTxt("Writing table");
		writeTable(prhs[0],fileName,format);
	}
	else
	{
		//mexWarnMsgTxt("Writing matrix");
		writeMatrix(prhs[0],fileName,format);
	}

	if (fileName!=NULL)
		mxFree(fileName);
	if (format!=NULL && strcmp(format,DEF_FORMAT)!=0)
		mxFree(format);
}
