package	mscanlib.gui;

import java.awt.*;
import java.io.File;
import javax.swing.*;
import javax.swing.plaf.basic.BasicFileChooserUI.*;

public final class MScanJFileChooser extends JFileChooser
{
	private static	int mSizeX=-1;
	private static	int mSizeY=-1;

	private File				mSelectedFile=null;
	private	boolean				mAppendOption=false;
	private boolean				mAppend=false;
	private MScanJFileFilter	mFileFilters[]=null;

	public MScanJFileChooser(String currentDir)
	{
		this(currentDir,false);
	}

	public MScanJFileChooser(String currentDir,boolean appendOption)
	{
		super(currentDir);
		this.mAppendOption=appendOption;

		if (mSizeX!=-1 && mSizeY!=-1)
			this.setPreferredSize(new Dimension(mSizeX,mSizeY));
	}

	public void setAppendOption(boolean appendOption)
	{
		this.mAppendOption=appendOption;
	}
	
	public boolean getAppend()
	{
		return(this.mAppend);
	}
	
	public File getSelectedFileWithExt()
	{
		return(this.mSelectedFile);
	}

	public void setFileFilters(String fileFormats[][])
	{
		this.mFileFilters=new MScanJFileFilter[fileFormats.length];
		for (int i=0;i<this.mFileFilters.length;i++)
		{
			this.mFileFilters[i]=new MScanJFileFilter(fileFormats[i][0],fileFormats[i][1],true);
			this.setFileFilter(this.mFileFilters[i]);
		}
	}

	public void setFileFilter(String fileFormat[])
	{
		MScanJFileFilter	fileFilters[]=null;
		
		if (this.mFileFilters!=null)
		{
			fileFilters=new MScanJFileFilter[this.mFileFilters.length+1];
			for (int i=0;i<this.mFileFilters.length;i++)
				fileFilters[i]=this.mFileFilters[i];
		}
		else
			fileFilters=new MScanJFileFilter[1];
		
		fileFilters[fileFilters.length-1]=new MScanJFileFilter(fileFormat[0],fileFormat[1],true);
		this.setFileFilter(fileFilters[fileFilters.length-1]);		

		this.mFileFilters=fileFilters;
	}

	public void addFileFilter(String fileFormat[])
	{
		MScanJFileFilter	fileFilters[]=null;
		
		if (this.mFileFilters!=null)
		{
			for (int i=0;i<this.mFileFilters.length;i++)
				this.removeChoosableFileFilter(this.mFileFilters[i]);
			
			fileFilters=new MScanJFileFilter[this.mFileFilters.length+1];
			for (int i=0;i<this.mFileFilters.length;i++)
				fileFilters[i+1]=this.mFileFilters[i];
		}
		else
			fileFilters=new MScanJFileFilter[1];
		
		fileFilters[0]=new MScanJFileFilter(fileFormat[0],fileFormat[1],true);
		for (int i=0;i<fileFilters.length;i++)
			this.setFileFilter(fileFilters[i]);		
		this.mFileFilters=fileFilters;
	}

	
	public int getFileFormat()
	{
		int fileFormat=-1;

		if (this.mFileFilters!=null && !this.getFileFilter().getDescription().toLowerCase().equals("all files"))
		{
			for (int i=0;i<this.mFileFilters.length;i++)
				if (((MScanJFileFilter)this.getFileFilter()).equals(this.mFileFilters[i]))
				{
					fileFormat=i;
					break;
				}
		}
		return(fileFormat);
	}

/*	public void setVisible(boolean visible)
	{
		if (this.isVisible())
		{
			mSizeX=this.getSize().width;
			mSizeY=this.getSize().height;
		}
		super.setVisible(visible);
	}*/

	public void cancelSelection()
	{
		mSizeX=this.getSize().width;
		mSizeY=this.getSize().height;
		super.cancelSelection();
	}

	public void approveSelection()
	{
		File					file=null;
		MScanJFileFilter	fileFilter=null;
		int						selectedOption=0;

		if (this.getDialogType()==JFileChooser.SAVE_DIALOG)
		{
			file=super.getSelectedFile();

			if (this.getFileFilter() instanceof MScanJFileFilter)
			{
				fileFilter=(MScanJFileFilter)this.getFileFilter();
				if (!file.getPath().toUpperCase().endsWith(fileFilter.mExtensions[0].toUpperCase()))
					file=new File(file.getPath()+fileFilter.mExtensions[0]);
			}
	
			if (this.getFileSelectionMode()!=MScanJFileChooser.DIRECTORIES_ONLY && file.exists())
				if (this.mAppendOption==false)
				{
					if (JOptionPane.showConfirmDialog(this,"The file " + file.getName() + " already exists!\nDo you want to replace it?",this.getDialogTitle(),JOptionPane.YES_NO_OPTION,JOptionPane.WARNING_MESSAGE)==JOptionPane.NO_OPTION)
						return;
				}
				else
				{
					if ((selectedOption=JOptionPane.showOptionDialog(this,"The file " + file.getName() + " already exists!",this.getDialogTitle(),JOptionPane.YES_NO_CANCEL_OPTION,JOptionPane.WARNING_MESSAGE,null,new String[]{"Overwrite","Append","Cancel"},"Cancel"))==JOptionPane.CANCEL_OPTION)
						return;
					if (selectedOption==1)
						this.mAppend=true;
				}
		
			this.mSelectedFile=file;
		}
		mSizeX=this.getSize().width;
		mSizeY=this.getSize().height;
		super.approveSelection();
	}
}