function [svm_model_params]=libsvm_optim(data,classes,svm_kernel,svm_model_params,svm_c_range,svm_gamma_range,optim_param,plot_grid)

if (svm_kernel==0)
	svm_c=svm_lin_optim(classes,data,svm_model_params,svm_c_range,optim_param,plot_grid);
elseif (svm_kernel==2)
	[svm_c,svm_gamma]=svm_rbf_optim(classes,data,svm_model_params,svm_c_range,svm_gamma_range,optim_param,plot_grid);
end;
		
if (svm_kernel==0)
    svm_model_params=[svm_model_params sprintf(' -c %f',svm_c)];
elseif (svm_kernel==2)    
    svm_model_params=[svm_model_params sprintf(' -c %f -g %f',svm_c,svm_gamma)];
end;
return;


%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [best_C]=svm_lin_optim(classes,data,model_params,c_range,optim_param,plot_grid)

C=c_range(1):c_range(3):c_range(2);

cv_acc = zeros(length(C),1); 
for i=1:length(C) 
	params=[model_params sprintf(' -c %f',10^C(i))];
	disp(['params=' params]);
	p=cv(params,data,classes,'kfold_strat',5);
	eval(['cv_acc(i)=p.' optim_param ';']);
    %cv_acc(i)=libsvmtrain(classes,data,params);      
	disp(sprintf('Parameters C-V performance: acc=%f, se=%f, sp=%f, ppv=%f, npv=%f, bac=%f, auc=%f, f1=%f, mcc=%f\n',p.acc,p.se,p.sp,p.ppv,p.npv,p.bac,p.auc,p.f1,p.mcc));
end;

[best_acc,idx]=max(cv_acc); 
best_C = 10^C(idx);

if plot_grid==1
	figure();
	plot(cv_acc);
	xlabel('log_1_0(C)');
	title('Linear kernel params C-V performance') ;
end;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [best_C,best_gamma]=svm_rbf_optim(classes,data,model_params,c_range,gamma_range,optim_param,plot_grid)

[C,gamma]=meshgrid(c_range(1):c_range(3):c_range(2),gamma_range(1):gamma_range(3):gamma_range(2)); 

cv_acc = zeros(numel(C),1); 
for i=1:numel(C) 
	params=[model_params sprintf(' -c %f -g %f',10^C(i),10^gamma(i))];
	disp(['params= ' params]);
	p=cv(params,data,classes,'kfold_strat',5);
	eval(['cv_acc(i)=p.' optim_param ';']);
    %cv_acc(i)=libsvmtrain(classes,data,params);
	disp(sprintf('Parameters C-V performance: acc=%f, se=%f, sp=%f, ppv=%f, npv=%f, bac=%f, auc=%f, f1=%f, mcc=%f\n',p.acc,p.se,p.sp,p.ppv,p.npv,p.bac,p.auc,p.f1,p.mcc));
end;

[best_acc,idx]=max(cv_acc); 
best_C = 10^C(idx);
best_gamma = 10^gamma(idx);

if plot_grid==1
	figure();
	surf(C,gamma,reshape(cv_acc,size(C)),'EdgeColor', 'None', 'facecolor', 'interp');
	view(2);
	colorbar;
	xlabel('log_1_0(C)');
	ylabel('log_1_0(\gamma)');
	title('RBF kernel params C-V performance') ;
end;