function [restored_data,delete_list]=missing_restore(data,method,param);

if nargin==1
	method='row_median';
end;

if nargin==2
	switch method
		case 'knn'
			nearest_nr=4;
		case 'svd'
			percent=0.2;
		case 'replace'
			value=0;
		case {'class_mean','class_median','class_min'}
			order=ones(1,size(data,2));
	end;
end;

if nargin==3
	switch method
		case 'knn'
			if param>0 & param<size(data,1)
				nearest_nr=param;
			else
				error('Niepoprawna liczba najbliszych ssiadw');
			end;
		case 'svd'
			if param>0 & param<1
				percent=param;
			else
				error('Niepoprawny procent cech wasnych');
			end;
		case 'replace'
			value=param;
		case {'class_mean','class_median','class_min'}
			order=param;
	end;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Znalezienie brakujcych wartoci%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
dimension=size(data,1);
vectors_count=size(data,2);
missing=isnan(data);
delete_list=[];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Rekonstrukcja wybran metod%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
switch lower(method)
	case 'replace'
		data(find(missing==1))=value;

	case 'data_min'
		nan_index=find(missing==1);
		not_nan_index=find(missing==0);

		min_value=min(data(not_nan_index));
		data(nan_index)=min_value;

	case 'nonzero_data_min'
		nan_index=find(missing==1);
		not_nan_index=find(missing==0);
		
		values=data(not_nan_index);
		values(find(values==0))=Inf;
		min_value=min(values);
		data(nan_index)=min_value;

	case 'column_min'
		for i=1:vectors_count
			nan_index=find(missing(:,i)==1);
			not_nan_index=find(missing(:,i)==0);
			values=data(not_nan_index,i);
			min_value=min(values);
			data(nan_index,i)=min_value;
		end;


	case 'nonzero_column_min'
		for i=1:vectors_count
			nan_index=find(missing(:,i)==1);
			not_nan_index=find(missing(:,i)==0);
			values=data(not_nan_index,i);
			values(find(values==0))=Inf;
			min_value=min(values);
			data(nan_index,i)=min_value;
		end;

	case 'row_mean'
		for i=1:dimension
			nan_index=find(missing(i,:)==1);
			if ~isempty(nan_index) && length(nan_index<vectors_count)
				not_nan_index=find(missing(i,:)==0);
				data(i,nan_index)=mean(data(i,not_nan_index));
			end;
		end;

	case 'row_median'
		for i=1:dimension
			nan_index=find(missing(i,:)==1);
			if ~isempty(nan_index) && length(nan_index<vectors_count)
				not_nan_index=find(missing(i,:)==0);
				data(i,nan_index)=median(data(i,not_nan_index));
			end;
		end;

	case 'row_min'
		for i=1:dimension
			nan_index=find(missing(i,:)==1);
			if ~isempty(nan_index) && length(nan_index<vectors_count)
				not_nan_index=find(missing(i,:)==0);
				data(i,nan_index)=min(data(i,not_nan_index));
			end;
		end;

	case 'class_mean'
		class_count=max(order);
		class_index=cell(1,class_count);
		class_vectors_count=zeros(1,class_count);
		for i=1:class_count
			class_index{i}=find(order==i);
			class_vectors_count(i)=length(class_index{i});
		end;

		for i=1:dimension
			nan_index=find(missing(i,:)==1);
			if ~isempty(nan_index) && length(nan_index)<vectors_count
				for j=1:class_count
					class_nan_index=intersect(class_index{j},nan_index);
					if ~isempty(class_nan_index) && length(class_nan_index)<class_vectors_count(j)
						value=mean(data(i,setdiff(class_index{j},nan_index)));
						data(i,class_nan_index)=value;
					end;
				end;
			end;
		end;

	case 'class_median'
		class_count=max(order);
		class_index=cell(1,class_count);
		class_vectors_count=zeros(1,class_count);
		for i=1:class_count
			class_index{i}=find(order==i);
			class_vectors_count(i)=length(class_index{i});
		end;

		for i=1:dimension
			nan_index=find(missing(i,:)==1);
			if ~isempty(nan_index) && length(nan_index)<vectors_count
				for j=1:class_count
					class_nan_index=intersect(class_index{j},nan_index);
					if ~isempty(class_nan_index) && length(class_nan_index)<class_vectors_count(j)
						value=median(data(i,setdiff(class_index{j},nan_index)));
						data(i,class_nan_index)=value;
					end;
				end;
			end;
		end;

	case 'class_min'
		class_count=max(order);
		class_index=cell(1,class_count);
		class_vectors_count=zeros(1,class_count);
		for i=1:class_count
			class_index{i}=find(order==i);
			class_vectors_count(i)=length(class_index{i});
		end;

		for i=1:dimension
			nan_index=find(missing(i,:)==1);
			if ~isempty(nan_index) && length(nan_index)<vectors_count
				for j=1:class_count
					class_nan_index=intersect(class_index{j},nan_index);
					if ~isempty(class_nan_index) && length(class_nan_index)<class_vectors_count(j)
						value=min(data(i,setdiff(class_index{j},nan_index)));
						data(i,class_nan_index)=value;
					end;
				end;
			end;
		end;

	case 'knn'
		t=clock;

		data=missing_restore(data,'row_median');

		d=zeros(dimension,1);
		m=zeros(dimension,1);

		m=mean(data,2);
		size(m)
		missing_rows=find(sum(missing,2)>0);
		neighbours=zeros(nearest_nr,1);

		for i=1:length(missing_rows)
			for j=1:dimension
				if j~=missing_rows(i)
					c=corrcoef(data(missing_rows(i),:),data(j,:));
					d(j)=1-c(1,2);
					if d(j)==0
						d(j)=Inf;
					end;
				else
					d(j)=Inf;
				end;
			end;
			[d,index]=sort(d);

%			figure;
%			plot(d);
			missing_index=find(missing(missing_rows(i),:)==1);
			missing_count=length(missing_index);
			for j=1:missing_count
				counter=0;
				value=0;
				w_sum=0;
				for k=1:dimension
					if missing(k,missing_index(j))==0
						w=1/d(k);
						value=value+w*data(k,missing_index(j))/m(k);
						w_sum=w_sum+w;
						counter=counter+1;
					end;
					if counter==nearest_nr
						break;
					end;
				end;
				if counter~=0
					data(missing_rows(i),missing_index(j))=m(missing_rows(i))*value/w_sum;
				end;
			end;
		end;
		%etime(clock,t)

	case 'svd'
		percent=param;
		
		data=missing_restore(data,'row_median');

		for iteration=1:10
			[u,s,v]=svd(data,0);
			eigen_nr=ceil(percent*size(s,1));

			data_estimation=u(:,1:eigen_nr)*s(1:eigen_nr,1:eigen_nr)*v(:,1:eigen_nr)';
			data(find(missing))=data_estimation(find(missing));
		end;
		
	case 'svd_test'
		percent=param;
		
		data=missing_restore(data,'row_median');
		[y x]=find(missing);
		
		for iteration=1:1
			%[u,s,v]=svd(data,0);
			%eigen_nr=ceil(percent*size(s,1));

			%EG=v(:,1:eigen_nr);
			[V S] = eig(data'*data);
			V=V(:,end:-1:1);
		
			% Wybor wykorzystywanych "genow wlasnych"
			EG = V(:,1:ceil(percent*length(diag(S))));
			size(EG)
			
			for i=1:dimension
				nan_index=x(find(y==i));
				if ~isempty(nan_index);
					index=setdiff([1:vectors_count],nan_index);
					%index=[1:vectors_count];
					
					% Wiersz macierzy M bez wartosci brakujacej
					M_row=data(i,index);
			
					%Regresja wiersza wzgledem "genow wlasnych" metoda OLS
					b=EG(index,:)\M_row';
			
					% Estymacja brakujacej wartosci na podstawie uzyskanego modelu
					for j=1:length(nan_index)
						%disp('aa');
						data(i,nan_index(j))=EG(nan_index(j),:)*b;
					end;
				end;	
			end;
		end;
		
	case 'svd_wagner'
		percent=param;
		
		data=missing_restore(data,'row_median');
		[y x]=find(missing);
		
		[V S] = eig(data'*data);
		V=V(:,end:-1:1);
		
		% Wybor wykorzystywanych "genow wlasnych"
		EG = V(:,1:ceil(percent*length(diag(S))));
		size(EG)
		% Rekonstrukcja brakujacych wartosci    
		for i=1:length(y)
			%disp(sprintf('%d %d %f',y(i),x(i),data(y(i),x(i))))

			% Wiersz macierzy M bez wartosci brakujacej
			M_row=data(y(i),[1:x(i)-1 x(i)+1:end]);
			%Regresja wiersza wzgledem "genow wlasnych" metoda OLS
			b=EG([1:x(i)-1 x(i)+1:end],:)\M_row';		
			% Estymacja brakujacej wartosci na podstawie uzyskanego modelu
			data(y(i),x(i))=EG(x(i),:)*b;
		end;

end;

restored_data=data;
		


