/* lsd_polynomial.h: Function prototype and global definitions for
 * various functions for doing one dimensional loess fitting.
 */
/* Rune Lyngs, April 2003 */
#ifndef _LOESS_H
#define _LOESS_H

/* Robust loess fitting iteration stop when the maximum change in any
 * weight between two consecutive iterations becomes less than
 * _LOESS_ROBUSTSTOP or when _LOESS_ROBUSTMAXITERATIONS iterations is
 * exceeded.
 */
#define _LOESS_ROBUSTSTOP 0.0001
#define _LOESS_ROBUSTMAXITERATIONS 100

typedef struct {
  int n;        /* Number of data points fitted */
  double *x;    /* x values of the n points */
  double *y;    /* y values of the n points */
  double *w;    /* Weights of the n points, if not NULL */
  double alpha; /* Neighbourhood defining parameter */
  int lambda;   /* Degree of polynomial used for fit */
  int q;        /* Number of points in neighbourhood (computable from
		 * alpha and n. */
} Loess;

/* Descriptions of functions precede the implementation of the
 * function in loess.c.
 */
Loess *loess(int n, double *x, double *y, double alpha, int lambda);
Loess *wloess(int n, double *x, double *y, double *w, double alpha,int lambda);
double loess_evaluate(Loess *l, double a);
Loess *loess_robustify(Loess *l);
double loess_nisd(Loess *l1, Loess *l2, double a, double b, int n);
void loess_changealpha(double alpha, Loess *l);
void loess_changelambda(int lambda, Loess *l);
void loess_free(Loess *l);

#endif
