function p=resampling_test(data,order,iterations,selection_method,pn)

if nargin<2
	error('The function needs al least two arguments');
elseif nargin==2
	iterations=1000;
	selection_method='anova';
	pn=0;
elseif nargin==3
	selection_method='anova';
	pn=0;
elseif nargin==4
	pn=0;
end;

if iterations==0
	iterations=1000;
end;

dimension=size(data,1);
vectors_count=size(data,2);

classes=unique(order);
classes_count=length(classes);

for i=1:classes_count
	classes(i)=length(find(order==classes(i)));
end;

p=zeros(dimension,1);

selection_method=lower(selection_method);
switch lower(selection_method)
	case {'satterthwaite'}
		class_index=[min(order) max(order)];
		stat=satterthwaite(data,order,class_index);
	case {'anova'}
		stat=anova_test(data,order);
	case {'anova2'}
		stat=mwanova(data,order);
	case {'pttest'}
		stat=t_testp(data);
end;

rand('state',0);
t0=clock;

if strcmp(selection_method,'mattest')==1
	disp(sprintf('Resampling stat: robust t (%d iterations)',iterations));
	
	p=mattest(data(:,find(order==min(order))),data(:,find(order==max(order))),'PERMUTE',iterations,'SHOWPLOT',false);
else
	waitbar_hnd=waitbar(0,'Wyznaczanie statystyki testowej...');

	disp(sprintf('Resampling stat: %s (iterations=%d,PN=%d)',selection_method,iterations,pn));
	if (pn==0)
		for i=1:iterations
			data=data(:,randperm(vectors_count));
			switch selection_method
				case {'satterthwaite'}
					current_stat=satterthwaite(data,order,class_index);
				case {'anova'}
					current_stat=anova_test(data,order);
				case {'anova2'}
					current_stat=mwanova(data,order);					
				case {'pttest'}
					current_stat=t_testp(data);
			end;
			p=p+(abs(current_stat)>=abs(stat));
			
			waitbar(i/iterations,waitbar_hnd);
		end;
		p=p/iterations;
	else
		full_stat=zeros(1,iterations*dimension);
		for i=1:iterations
			data=data(:,randperm(vectors_count));
			switch selection_method
				case {'satterthwaite'}
					current_stat=satterthwaite(data,order,class_index);
				case {'anova'}
					current_stat=anova_test(data,order);
				case {'anova2'}
					current_stat=mwanova(data,order);					
				case {'pttest'}
					current_stat=t_testp(data);					
			end;
			full_stat((i-1)*dimension+1:i*dimension)=current_stat;
			
			waitbar(i/iterations,waitbar_hnd);
		end;
		p=zeros(dimension,1);
		for i=1:dimension
			p(i)=sum(abs(full_stat)>=abs(stat(i)))/(iterations*dimension);
		end;
	end;
	
	delete(waitbar_hnd);
end;
%etime(clock,t0)
return;
