function values_table(action,rows)

if nargin==0
	action='create';
	rows=[];
elseif nargin==1
	rows=[];	
end;

import com.mathworks.mwt.*;
import java.awt.* ;
import javax.swing.*;
import javax.swing.table.*;

table_id='ValueTable';
table_name='Wartoci';

Array_id=findobj(get(0,'Child'),'Tag','ArrayView');
if ~isempty(Array_id)
    frame=findobj('Tag',table_id);
    
    switch action
		case 'create'
			if ~isempty(frame)
				delete(frame);
            end
            
            set(Array_id,'Pointer','watch');
            pause(0.01);

            data=get(Array_id,'UserData');
            config=get(findobj(Array_id,'Tag','FileOpen'),'UserData');

            anames=get(findobj(Array_id,'Tag','SelectedArrayNameEdit'),'UserData');
            aorder=get(findobj(Array_id,'Tag','ArrayClassAxes'),'UserData');
            acnames=get(findobj(Array_id,'Tag','ArrayClassPop'),'UserData');

            gnames=get(findobj(Array_id,'Tag','SelectedGeneNameEdit'),'UserData');
            gorder=get(findobj(Array_id,'Tag','GeneClassAxes'),'UserData');
            gcnames=get(findobj(Array_id,'Tag','GeneClassPop'),'UserData');

            %%%%%%%%%%%%%%%%
            %Tworzenie okna%
            %%%%%%%%%%%%%%%%
            frame=figure('Units','pixels', ...
                    'Color',[0.8 0.8 0.8], ...
                    'Visible','on',...
                    'MenuBar','none', ...
                    'NumberTitle','off', ...
                    'Resize','off',...
                    'Name',table_name, ...
                    'KeyPressFcn',@figure_kb_cb,...
                    'Tag',table_id);
                
            table=create_table(frame);
            fill_table(table);
			set(frame,'UserData',table);
            
            control_panel=create_control_panel(frame);
            
            main_panel=JPanel;
			main_panel.setLayout(BorderLayout);

			table_panel=JScrollPane;
			scrollayout=ScrollPaneLayout;
			table_panel.setLayout(scrollayout);
			viewport=get(scrollayout,'ViewPort');
			viewport.setView(table);
            table.requestFocus
            
			main_panel.add(table_panel,BorderLayout.CENTER);
			main_panel.add(control_panel,BorderLayout.SOUTH);

			[size_x,size_y]=get_table_size(table);
            screen_size=get(0,'Screensize');
            size_y=min(size_y,screen_size(4)-75);
			set(frame,'Position',[5 (screen_size(4)-size_y-30) size_x size_y])
			
			javacomponent(main_panel,[0 0 size_x size_y],frame);
			
			set_selection(table);
			set(Array_id,'Pointer','arrow');
		case 'reorder'
			fill_table(get(frame,'UserData'));

        case 'resize'
			resize_frame(frame);
            
        case 'remove'
            if ~isempty(rows)		
				remove_rows(frame,rows);
			end;
            
        case 'set_selection'
			set_selection(get(frame,'UserData'));
    end;
end;
return;


%%%%%%%%%%%%%%%%%%
%Tworzenie tabeli%
%%%%%%%%%%%%%%%%%%
function table=create_table(frame)

import javax.swing.*;
import javax.swing.table.*;

Array_id=findobj(0,'Tag','ArrayView');
anames=get(findobj(Array_id,'Tag','SelectedArrayNameEdit'),'UserData');
data=get(Array_id,'UserData');
 
table_model=DefaultTableModel([{'Cechy'};anames],size(data,1));
table=JTable(table_model);
table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
%table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
%table.setCellSelectionEnabled(0);
table.setColumnSelectionAllowed(1);
            
%h=handle(table,'callbackproperties');
%set(h,'MousePressedCallback',{@mouse_cb,table,frame});

columnModel=table.getColumnModel;
columns_count=table.getColumnCount;

column=columnModel.getColumn(0);
column.setPreferredWidth(150);
renderer=table.getTableHeader.getDefaultRenderer;
renderer.setHorizontalAlignment(SwingConstants.LEFT);
column.setCellRenderer(renderer);

for i=1:columns_count-1
    column=columnModel.getColumn(i);
    column.setPreferredWidth(80);
    renderer=DefaultTableCellRenderer;
    renderer.setHorizontalAlignment(SwingConstants.CENTER);
    column.setCellRenderer(renderer);
end;
return;

%%%%%%%%%%%%%%%%%%%%
%Wypelnienie tabeli%
%%%%%%%%%%%%%%%%%%%%
function fill_table(table)

Array_id=findobj(0,'Tag','ArrayView');
data=get(Array_id,'UserData');

gnames=get(findobj(Array_id,'Tag','SelectedGeneNameEdit'),'UserData');

table_model=table.getModel;
rows_count=table.getRowCount;
columns_count=table.getColumnCount;

for i=1:rows_count
    table_model.setValueAt(gnames{i},i-1,0);
    for j=1:columns_count-1
        %table_model.setValueAt(sprintf('%.2f',data(i,j)),i-1,j);
        table_model.setValueAt(sprintf('%f',data(i,j)),i-1,j);
    end;
end;
return;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Tworzenie panelu kontrolnego%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function control_panel=create_control_panel(frame)

import javax.swing.*;
import java.awt.* ;

table=get(frame,'UserData');

control_panel=JPanel;
box_layout=BoxLayout(control_panel,BoxLayout.X_AXIS);
control_panel.setLayout(box_layout);

control_panel.add(Box.createRigidArea(Dimension(2,30)));

control_panel.add(Box.createHorizontalGlue);

button=handle(JButton('Zaznacz'),'CallbackProperties');
set(button,'ActionPerformedCallback',{@buttons_cb,button,table,frame,'Zaznacz'});
control_panel.add(button);


control_panel.add(Box.createRigidArea(Dimension(5,30)));

button=handle(JButton('Zastosuj'),'CallbackProperties');
set(button,'ActionPerformedCallback',{@buttons_cb,button,table,frame,'Zastosuj'});
control_panel.add(button);
return;

%%%%%%%%%%%%%%%%%
%Skalowanie okna%
%%%%%%%%%%%%%%%%%
function [size_x,size_y]=get_table_size(table)

columns_count=table.getColumnCount;
rows_count=table.getRowCount;

table_width=150+(columns_count-1)*80;
table_height=(rows_count+1)*table.getRowHeight;

screen_size=get(0,'Screensize');
size_x=min(table_width+10,screen_size(3)-10);
size_y=min(table_height+40,screen_size(4)-25);
return;

%%%%%%%%%%%%%
%Remove rows%
%%%%%%%%%%%%%
function remove_rows(frame,rows)

table=get(frame,'UserData');
			
rows=sort(rows,'descend');
table_model=table.getModel;
for i=1:size(rows,1)
    if rows(i,1)~=0
        table_model.removeRow(rows(i,1)-1);
    elseif rows(i,2)~=0
        table_model.removeColumn(rows(i,2));
    end;
end;
table.updateUI;
drawnow;
                
resize_frame(frame);
return;

%%%%%%%%%%%%%%%
%Set selection%
%%%%%%%%%%%%%%%
function set_selection(table)

Array_id=findobj(0,'Tag','ArrayView');
config=get(findobj(Array_id,'Tag','FileOpen'),'UserData');
			
if config.view_struct.selected_gene>0 && config.view_struct.selected_array>0
	table.changeSelection(config.view_struct.selected_gene-1,config.view_struct.selected_array,false,false);
	table.repaint;
	drawnow;
end;
return;

%%%%%%%%%%
%Callback%
%%%%%%%%%%
function buttons_cb(varargin)

button=varargin{3};
table=varargin{4};
frame=varargin{5};
cmd=varargin{6};

Array_id=findobj(get(0,'Child'),'Tag','ArrayView');
if ~isempty(Array_id)

	data=get(Array_id,'UserData');
	config=get(findobj(Array_id,'Tag','FileOpen'),'UserData');
	anames=get(findobj(Array_id,'Tag','SelectedArrayNameEdit'),'UserData');
	aorder=get(findobj(Array_id,'Tag','ArrayClassAxes'),'UserData');
	acnames=get(findobj(Array_id,'Tag','ArrayClassPop'),'UserData');
	classes=unique(aorder);

	gnames=get(findobj(Array_id,'Tag','SelectedGeneNameEdit'),'UserData');
	gorder=get(findobj(Array_id,'Tag','GeneClassAxes'),'UserData');
	gcnames=get(findobj(Array_id,'Tag','GeneClassPop'),'UserData');

	table_model=table.getModel;
	columns_count=table.getColumnCount;
	rows_count=table.getRowCount;

	switch char(cmd)
		case 'Zastosuj'
			for i=1:rows_count
				for j=1:columns_count-1
					value=str2num(table_model.getValueAt(i-1,j));
					if ~isempty(value)
						data(i,j)=value;
					else
						data(i,j)=NaN;
					end;
					table_model.setValueAt(sprintf('%f',data(i,j)),i-1,j);
				end;
			end;

			set(Array_id,'UserData',data);
			mstat_cb('update_missing');
			mstat_cb('prepare_data');
			mstat_cb('redraw');
			mstat_cb('do_menu');
			mstat_cb('do_gui');

		case 'Zaznacz'
			gene_index=table.getSelectedRow;
			array_index=table.getSelectedColumn;
			if gene_index>=0 && array_index>0
				gene_nr=gene_index+1;
				array_nr=array_index;

				config.view_struct.selected_array=array_nr;
				config.view_struct.selected_gene=gene_nr;
				set(findobj(Array_id,'Tag','FileOpen'),'UserData',config);
				mstat_cb('set_selected_array');
				mstat_cb('set_selected_gene');
			end;
	end;
end;
return;
