#!/usr/bin/python
from sys import argv, stdin, stderr, stdout, exit
from os import execl, popen2, environ
from types import StringType, ListType, DictType, TupleType
from operator import add, concat, and_
from getopt import GetoptError
try:
  from getopt import gnu_getopt as getopt
except ImportError:
  from getopt import getopt
from getopt import GetoptError

# The getopt module does not support options taking optional arguments. This
# class provides an extended getopt to allow optional arguments.
class MyGetopt:
  "An option parser allowing optional arguments"
  # Constants defining option type
  NONE, REQUIRED, OPTIONAL = range(0, 3)
  def __init__(self, short_options = None, long_options = None):
    self.long_options = {}
    self.short_options = {}
    if environ.has_key("POSIXLY_CORRECT"):
      self.posix = True
    else:
      self.posix = False
    self.add_options(short_options, long_options)
  def add_options(self, short_options, long_options):
    # Add or change short options in dictionary
    if short_options:
      if type(short_options) != StringType:
        stderr.write("Short options should be a string\n")
        raise GetoptError
      if short_options[0] == '+':
        self.posix = True
        i = 1
      else:
        i = 0
      while i < len(short_options):
        # Check whether this option takes (optional) argument
        if i + 1 < len(short_options) and short_options[i + 1] == ':':
          # Check whether the argument is optional
          if i + 2 < len(short_options) and short_options[i + 2] == ':':
            self.short_options[short_options[i]] = self.OPTIONAL
            i += 3
          else:
            self.short_options[short_options[i]] = self.REQUIRED
            i += 2
        else:
          self.short_options[short_options[i]] = self.NONE
          i += 1
    # Add or change long options in trie
    if long_options:
      if type(long_options) != ListType:
        stderr.write("Long options should be a list of strings\n")
        raise GetoptError
      for option in long_options:
        if type(option) != StringType:
          stderr.write("Long options should be a list of strings\n")
          raise GetoptError
        try:
          # Check whether this options takes (optional) argument
          argument = self.NONE
          if option[-1] == '=':
            argument = self.REQUIRED
            option = option[:-1]
            # Check whether the argument is optional
            if option[-1] == '=':
              argument = self.OPTIONAL
              option = option[:-1]
          # Find options position in trie
          if self.long_options.has_key(option[0]):
            # Other long options starting with same symbol
            trie = self.long_options[option[0]]
            oldtrie = self.long_options
            i = 1
            while type(trie) == DictType and i < len(option):
              if trie.has_key(option[i]):
                # This option is not uniquely defined until further down
                oldtrie = trie
                trie = trie[option[i]]
                i += 1
              else:
                # No other options with prefix read so far
                trie[option[i]] = (option, argument)
                break
            else:
              # Either trie is just a pair representing an option or we
              # have read the entire option
              if type(trie) != DictType:
                # We have read a prefix that previously uniquely determined
                # the option
                if option == trie[0]:
                  # Redefinition of old option
                  oldtrie[option[i - 1]] = (option, argument)
                else:
                  # New option sharing a prefix with an old option
                  j = min(len(option), len(trie[0]))
                  while i < j and option[i] == trie[0][i]:
                    # Prefix is still shared
                    oldtrie[option[i - 1]] = {option[i]: None}
                    oldtrie = oldtrie[option[i - 1]]
                    i += 1
                  # We have reached the end of the shared prefix
                  if i >= j:
                    # One option is a prefix of the other
                    if len(option) < len(trie[0]):
                      # New option is prefix of old option
                      oldtrie[option[i - 1]] = {'': (option, argument), trie[0][i]: trie}
                    else:
                      # Old option is a prefix of new option
                      oldtrie[option[i - 1]] = {'': trie, option[i]: (option, argument)}
                  else:
                    # Two options only share a proper prefix
                    oldtrie[option[i - 1]] = {option[i]: (option, argument), trie[0][i]: trie}
              else:
                # New option is a prefix of more than one existing options
                oldtrie[option[i - 1]][''] = (option, argument)
          else:
            # New initial symbol encountered
            self.long_options[option[0]] = (option, argument)
        except IndexError:
          stderr.write("A long option should have a non-empty name\n")
          raise GetoptError
  def parse(self, argv):
    # Check that arguments are in the right form
    if type(argv) != ListType:
      stderr.write("Arguments has to be passed as a list of strings\n")
      raise ValueError
    for arg in argv:
      if type(arg) != StringType:
        stderr.write("Arguments has to be passed as a list of strings\n")
        raise ValueError
    fails = len(argv) * [False]
    def parse_remainder(argv, current = None):
      # Parse remainder of arguments in argv, with remaining short options
      # block current
      if current:
        # We are in the process of parsing short options
        if self.short_options.has_key(current[0]):
          # Next symbol is a valid short option
          if len(current) == 1:
            # Last short option in block
            if self.short_options[current] == self.NONE or self.short_options[current] == self.OPTIONAL and (len(argv) == 0 or len(argv[0]) > 0 and argv[0][0] == '-'):
              # Try to parse with no option argument
              result = parse_remainder(argv)
              if type(result) == TupleType:
                # Successful parse
                result[0].insert(0, ('-' + current, None))
                return result
              elif self.short_options[current] == self.NONE:
                # Unsuccessful parse
                return result
            # Try to parse with option argument
            if argv:
              result = parse_remainder(argv[1:])
              if type(result) == TupleType:
                # Successful parse
                result[0].insert(0, ('-' + current, argv[0]))
              return result
            # Unsuccessful parse
            return "No argument to option -" + current + "\n"
          else:
            # More symbols in block
            if self.short_options[current[0]] != self.REQUIRED:
              # Try to parse more short options
              result = parse_remainder(argv, current[1:])
              if type(result) == TupleType:
                # Successful parse
                result[0].insert(0, ('-' + current[0], None))
                return result
              elif self.short_options[current[0]] == self.NONE:
                # Unsuccessful parse
                return result
            # Try to parse with remainder of block argument to option
            result = parse_remainder(argv)
            if type(result) == TupleType:
              # Successful parse
              result[0].insert(0, ('-' + current[0], current[1:]))
              return result
            # Unsuccessful parse
            return result
        else:
          # Next symbol is not a valid short option
          return "Unrecognised option -" + current[0] + "\n"
      else:
        # Start parsing next block in argv
        if argv:
          # Do we know for certain that this will be in vain?
          if fails[len(argv) - 1]:
            return None
          if argv[0] and argv[0][0] == '-':
            # Next argument is an option specification
            if len(argv[0]) == 1:
              # Well, actually it indicates end of options
              return ([], argv[1:])
            if argv[0][1] == '-':
              # Next argument is a long option
              if len(argv[0]) == 2:
                # Well, actually it indicates end of options
                return ([], argv[1:])
              # Determine option
              trie = self.long_options
              tentative = None
              i = 0
              while type(trie) == DictType and i + 2 < len(argv[0]):
                if argv[0][i + 2] == '=' and trie.has_key('') and trie[''][1] != self.NONE:
                  # This could be an option followed by an argument
                  tentative = ("--" + trie[''][0], argv[0][i + 3:])
                if not trie.has_key(argv[0][i + 2]):
                  if tentative != None:
                    # We already passed the right split into option and
                    # argument previously
                    result = parse_remainder(argv[1:])
                    if type(result) == TupleType:
                      # Successful parse
                      result[0].insert(0, tentative)
                    else:
                      # Unsuccessful parse
                      fails[len(argv) - 1] = True
                    return result
                  else:
                    # Unknown option encountered
                    fails[len(argv) - 1] = True
                    return "Unknown long option, " + argv[0] + ", encountered\n"
                trie = trie[argv[0][i + 2]]
                i += 1
              if type(trie) == DictType:
                # Entire option read without reaching a leaf of the trie
                if i + 2 == len(argv[0]) and trie.has_key(''):
                  # Option is valid, its just a prefix of another option
                  trie = trie['']
                else:
                  # Option not uniquely specified
                  fails[len(argv) - 1] = True
                  return "Unknown long option, " + argv[0] + ", encountered\n"
              else:
                # Make sure that the option given is indeed a prefix of the
                # unique option matching so far
                j = min(len(argv[0]) - 2, len(trie[0]))
                while i < j and argv[0][i + 2] == trie[0][i]:
                  i += 1
                if i < len(argv[0]) - 2:
                  # Anything left in the option must be an argument
                  if argv[0][i + 2] != '=':
                    # No match
                    fails[len(argv) - 1] = True
                    return "Unknown long option, " + argv[0] + ", encountered\n"
                  elif trie[1] == self.NONE:
                    fails[len(argv) - 1] = True
                    return "Long option --" + trie[0] + " does not take an argument\n"
                  else:
                    # Long option with argument identified
                    result = parse_remainder(argv[1:])
                    if type(result) == TupleType:
                      # Successful parse
                      result[0].insert(0, ("--" + trie[0], argv[0][i + 3:]))
                    else:
                      # Unsuccessful parse
                      fails[len(argv) - 1] = True
                    return result
              # We have matched a long option with no argument glued to it
              if trie[1] == self.NONE or trie[1] == self.OPTIONAL and (len(argv) == 1 or len(argv[1]) > 0 and argv[1][0] == '-'):
                # Try to parse with no option argument
                result = parse_remainder(argv[1:])
                if type(result) == TupleType:
                  # Successful parse
                  result[0].insert(0, ('--' + trie[0], None))
                  return result
                elif trie[1] == self.NONE:
                  # Unsuccessful parse
                  fails[len(argv) - 1] = True
                  return result
              # Try to parse with option argument
              if len(argv) > 1:
                result = parse_remainder(argv[2:])
                if type(result) == TupleType:
                  # Successful parse
                  result[0].insert(0, ('--' + trie[0], argv[1]))
                else:
                  # Unsuccessful parse
                  fails[len(argv) - 1] = True
                return result
              # Unsuccessful parse
              fails[len(argv) - 1] = True
              return "No argument to option --" + trie[0] + "\n"
            else:
              # Argument is a short option block
              result = parse_remainder(argv[1:], argv[0][1:])
              if type(result) != TupleType:
                # Unsuccessful parse
                fails[len(argv) - 1] = True
              return result
          else:
            # Next argument is not an option
            result = parse_remainder(argv[1:])
            if type(result) == TupleType:
              # Successful parse
              result[1].insert(0, argv[0])
            else:
              # Unsuccessful parse
              fails[len(argv) - 1] = True
            return result
        else:
          # End of arguments reached
          return ([], [])
    # Parse arguments according to options stored in this object
    result = parse_remainder(argv)
    if type(result) == TupleType:
      # Successful parse
      return result
    else:
      # Unsuccessful parse
      raise GetoptError(result)

# Split s according to separator but keep separator occurences as part of the
# list
def mysplit(s, separator):
  if separator == []:
    # Do manual split so that we can remember full separators
    index = len(s) - len(s.lstrip())
    t = [s[:index]]
    s = s[index:]
    while s:
      u = s.split(None, 1)
      if len(u) == 2:
        index1, index2 = len(u[0]), len(s) - len(u[1])
        t.extend([s[:index1], s[index1: index2]])
        s = s[index2:]
      else:
        index = len(u[0])
        t.extend([s[:index], s[index:]])
        break
    return t
  else:
    def onesplit(s, separators):
      if separators == []:
        return [s]
      t = s.split(separators[0])
      s = []
      for block in t:
        s.append(onesplit(block, separators[1:]))
      return reduce(lambda x, y: x + [separators[0]] + y, s[1:], s[0])
    return [""] + onesplit(s, separator)

# Concatenate the strings in the list s
def myjoin(s):
  return reduce(concat, s)

# Split the rows of data into those that fail all relations in relations, and
# those that don't
def check_relations(data, relations):
  fails, meets = [], []
  def check_line(line):
    for r in relations:
      try:
        x = (line[2 * r[0] + 2], r[2])
      except IndexError:
        stderr.write("WARNING: relation specified for column " + `r[0] + 1` + " that does not exist\nin input data line " + `line[0] + 1` + ":\n " + myjoin(line[1:]) + ";\nignoring")
        continue
      for operator in r[1]:
        x = operator(x)
      if x:
        return 1
    return 0
  for line in data:
    if check_line(line):
      # line meets at least one relation
      meets.append(line)
    else:
      fails.append(line)
  return fails, meets

# Normalise data where both experiments are contained in the same file; data
# should be the list of measurement lines read from the file and subjected
# to mysplit, i.e. a list of lists, c1 and c2 should be the two columns
# containing the measurements, ignore should be the relations specifying which
# measurements that should be left out of the normalisation, grouping should
# be a list of columns that data is grouped by (where each group is normalised
# independently), and options should be a text string containing the options
# to istvan.
def normalise_onefile(data, c1, c2, ignore, grouping, options):
  # Remove rows for which an ignore relation holds
  data, ignored = check_relations(data, ignore)
  if grouping:
    # Separate data into groups
    groups = {}
    for line in data:
      # Build key for this data line
      key = ()
      for c in grouping:
        try:
          key += (line[2 * c + 2], )
        except IndexError:
          key += (None, )
          stderr.write("WARNING: column " + `c + 1` + " specified for grouping key does not exist in line\n " + myjoin(line[1:]) + "\n")
      if groups.has_key(key):
        groups[key].append(line)
      else:
        groups[key] = [line]
  else:
    groups = {(): data}
  # Normalise the data group by group
  for data in groups.values():
    # Extract the two measurement from each line
    try:
      values = map(lambda x: (x[2 * c1 + 2], x[2 * c2 + 2]), data)
    except IndexError:
      for i in data:
        if 2 * c1 + 2 >= len(i):
          stderr.write("ERROR: column " + `c1 + 1` + " that should contain a measurement from the first\nexperiment does not exist in line\n" + myjoin(i[1:]) + "\n")
          exit(1)
        if 2 * c2 + 2 >= len(i):
          stderr.write("ERROR: column " + `c2 + 1` + " that should contain a measurement from the second\nexperiment does not exist in line\n" + myjoin(i[1:]) + "\n")
          exit(1)
    # Now perform the normalisation
    input, output = popen2("./istvan" + options)
    for line in values:
      input.write(line[0] + " " + line[1] + "\n")
    input.close()
    for i in range(0, len(data)):
      data[i][2 * c1 + 2], data[i][2 * c2 + 2] = output.readline().split()
    output.close()

# Match the genes in the two data sets represented by the pair data. The
# match pair indicates which columns form the key in each of the data sets,
# the eliminate and ignore pairs holds the relations by which genes are
# eliminated or ignored, respectively, and retain_singletons is a boolean that
# specifies whether a gene occurring in one data set but not in the other
# should be ignored or eliminated.
def match_genes(data, match, eliminate, ignore, retain_singletons):
  eliminated = ([], [])
  if match == [[], []]:
    # Match genes by ordering
    if len(data[0]) != len(data[1]):
      stderr.write("WARNING: the two data sets have different number of genes, " + `len(data[0])` + " and " + `len(data[1])` + ".\nI'll continue matching the first " + `min(len(data[0]), len(data[1]))` + " genes in the two files,\nbut probably something is wrong.\n")
      if not retain_singletons:
        for i in range(len(data[1]), len(data[0])):
          # Tabulate trailing lines from first data set
          eliminated[1].append(data[0][i][0])
        for i in range(len(data[0]), len(data[1])):
          # Tabulate trailing lines from second data set
          eliminated[1].append(data[1][i][0])
      data = map(lambda x, y: (x, y), data[0], data[1])[:min(len(data[0]), len(data[1]))]
    else:
      data = map(lambda x, y: (x, y), data[0], data[1])
  else:
    # Generate key for each line in first data set
    data0 = {}
    for line in data[0]:
      key = ()
      for c in match[0]:
        key += (line[2 * c + 2], )
      if data0.has_key(key):
        # Same id encountered multiple times
        stderr.write("WARNING: same gene id, " + `key` + ", encountered multiple times in first data set.\nIgnoring all but the first occurrence.\n")
        if not retain_singletons:
          eliminated[0].append(line[0])
      else:
        data0[key] = line
    # Generate key for each line in second data set
    data1 = {}
    for line in data[1]:
      key = ()
      for c in match[1]:
        key += (line[2 * c + 2], )
      if data1.has_key(key):
        # Same id encountered multiple times
        stderr.write("WARNING: same gene id, " + `key` + ", encountered multiple times in second data set.\nIgnoring all but the first occurrence.\n")
        if not retain_singletons:
          eliminated[1].append(line[0])
      else:
        data1[key] = line
    # Find matching keys
    data = []
    # Run through keys in first data set
    for line in data0.items():
      if data1.has_key(line[0]):
        # This gene has a match in the second data set
        data.append((line[1], data1[line[0]]))
        del data1[line[0]]
      else:
        if not retain_singletons:
          eliminated[0].append(line[1][0])
    if not retain_singletons:
      # Insert lines still remaining in data1, i.e. lines with keys not
      # occurring in the first data set, in ignored
      for line in data.values():
        eliminated[1].append(line[0])
  # We now have the matching genes stored in data; determine which should be
  # eliminated and which should be ignored
  def check_line(line, relations):
    # Determine whether line meets at least one of relations
    for r in relations:
      try:
        x = (line[2 * r[0] + 2], r[2])
      except IndexError:
        stderr.write("WARNING: relation specified for column " + `r[0] + 1` + " that does not exist\nin input data line " + `line[-1] + 1` + ":\n " + myjoin(line[1:]) + ";\nignoring")
        continue
      for operator in r[1]:
        x = operator(x)
      if x:
        return 1
    return 0
  i = 0
  while i < len(data):
    if check_line(data[i][0], eliminate[0]):
      # Gene meets an eliminate criteria in first data set
      eliminated[0].append(data[i][0][0])
      if not retain_singletons or check_line(data[i][1], eliminate[1]):
        # And it is not a keeper in the second data set either
        eliminated[1].append(data[i][1][0])
      data = data[:i] + data[i + 1:]
    elif check_line(data[i][1], eliminate[1]):
      # Gene meets an eliminate criteria in second data set
      eliminated[1].append(data[i][1][0])
      if not retain_singletons:
        # Forces elimination in first data set as well
        eliminated[0].append(data[i][0][0])
      data = data[:i] + data[i + 1:]
    elif check_line(data[i][0], ignore[0]) or check_line(data[i][1], ignore[1]):
      # Gene meets an ignore criteria in one of the data sets
      data = data[:i] + data[i + 1:]
    else:
      i += 1
  return data, eliminated

# Normalise data where the experiments are taken from two different
# files; data should be the lists of measurement lines read from the
# two files and subjected to mysplit, i.e. a list of lists, columns
# should be the two columns containing the measurements, grouping
# should be lists of columns that data is grouped by (where each group
# is normalised independently), and options should be a text string
# containing the options to istvan. Except for options, all parameters
# should be pairs.
def normalise_twofiles(data, columns, grouping, options):
  if grouping:
    # Separate data into groups
    groups = {}
    for line in data:
      # Build key for this data line
      key = ()
      # Build first data set part of key
      for c in grouping[0]:
        try:
          key += (line[0][2 * c + 2], )
        except IndexError:
          key += (None, )
          stderr.write("WARNING: column " + `c + 1` + " specified for grouping key does not exist in line\n " + myjoin(line[1:]) + "\nin first data set")
      # Build second data set part of key
      for c in grouping[1]:
        try:
          key += (line[1][2 * c + 2], )
        except IndexError:
          key += (None, )
          stderr.write("WARNING: column " + `c + 1` + " specified for grouping key does not exist in line\n " + myjoin(line[1:]) + "\nin second data set")
      if groups.has_key(key):
        groups[key].append(line)
      else:
        groups[key] = [line]
  else:
    groups = {(): data}
  # Normalise the data group by group
  for data in groups.items():
    # Extract the two measurement from each line
    try:
      values = map(lambda x: (x[0][2 * columns[0] + 2], x[1][2 * columns[1] + 2]), data[1])
    except IndexError:
      for i in data[1]:
        if 2 * columns[0] + 2 >= len(i[0]):
          stderr.write("ERROR: column " + `columns[0] + 1` + " that should contain a measurement from the first\nsample does not exist in line\n" + myjoin(i[0][1:]) + "\n")
          exit(1)
        if 2 * columns[1] + 2 >= len(i[1]):
          stderr.write("ERROR: column " + `columns[1] + 1` + " that should contain a measurement from the second\nsample does not exist in line\n" + myjoin(i[1][1:]) + "\n")
          exit(1)
    # Now perform the normalisation
    input, output = popen2("./istvan" + options)
    for line in values:
      input.write(line[0] + " " + line[1] + "\n")
    input.close()
    values = []
    for i in range(0, len(data[1])):
      line = output.readline()
      if line == "":
        if grouping != [[], []]:
          stderr.write("WARNING: Failed to normalise group identified with key " + `data[0]` + "\n")
          break
        else:
          stderr.write("ERROR: Failed to normalise data\n")
          exit(1)
      values.append(line.split())
    else:
      for i in range(0, len(values)):
        data[1][i][0][2 * columns[0] + 2], data[1][i][1][2 * columns[1] + 2] = values[i]
    output.close()

def print_usage(s, exit_value = 0):
  print "Usage: " + s + " [options] [infile [outfile [infile2 [outfile2]]]]"
  print "The script reads microarray data and uses the istvan program to"
  print "perform pairwise normalisation it. The data is read from infile"
  print "(standard input if no input file is specified) and the normalised"
  print "data is written to outfile (standard output if no output file is"
  print "specified). If the --separate-files option is specified, the second"
  print "sample is read from infile2 and written to outfile2. The normalised"
  print "data output will be an exact copy of the input data except for the"
  print "changes in measurement data columns due to normalisation and the"
  print "absence of eliminated genes, see the --eliminate option. In general,"
  print "long options specify how the input should be parsed to extract"
  print "measurements to normalise, while short options specify the method used"
  print "to normalise the data."
  print "Legal options are (capital letters following an option denotes a"
  print "mandatory argument, capital letters in square brackets denotes an"
  print "optional argument):"
  print " -? -h -H --help: Print this information and stop"
  print " -a: Normalise in A/M (mean/ration) transformed space; default is to"
  print "     normalise second sample intensities to first sample intensities"
  print " -c: Normalising in A/M transformed space usually results in"
  print "     modifications of measurements for both samples. The -c option"
  print "     changes this behavious such that only the measurements of the"
  print "     second sample measurements are modified. If this normalisation"
  print "     is part of a multi sample experiment where all samples are"
  print "     normalised to a central reference sample, use the -c option to"
  print "     specify that the first sample is the reference sample. This"
  print "     option does not set A/M transformed normalisation, so it needs"
  print "     to be used in combination with the -a option"
  print " -e: Do not work in log transformed space; default behaviour is to"
  print "     replace all measurements with their (base e) logarithm"
  print " -xN: Exclude the N genes with lowest average intensity and the"
  print "      N genes with highest average intensity (average taken in log"
  print "      space unless the -e option is specified) before inferring"
  print "      normalisation function for each group (see the --group-by"
  print "      option) of genes normalised"
  print " -g: Global normalisation, i.e. use all genes with equal weight for"
  print "     normalisation function inference"
  print " -b[X]: Use iterated global normalisation to determine gene weights"
  print "        for normalisation function inference; the new weight is the"
  print "        bisquare function of the distance to the previous estimate"
  print "        relative to X times the median distance, where the default"
  print "        value of X is 6"
  print " -t[X]: Use iterated global normalisation to determine gene weights"
  print "        for normalisation function inference; the new weight is the"
  print "        Gaussian function of the distance to the previous estimate"
  print "        relative to X times the sample variance, where the default"
  print "        value of X is 1"
  print " -r[N]: Use invariant rank selection with threshold N to compute an"
  print "        invariant set; the default value of N is 15"
  print " -i[F]: Use Iterated invariant rank selection to compute an invariant"
  print "        set; in each iteration a gene is retained if its rank"
  print "        difference is at most F times the number of remaining genes;"
  print "        the default value of F is 0.02"
  print " -jF: Set the threshold percentage for iterated invariant rank"
  print "      selection to be an interpolation between the rank specified by"
  print "      the -i option (or the default value) at low expression values"
  print "      and F at high expression values; this option does not set the"
  print "      invariant set method so it needs to be used in combination with"
  print "      the -i option"
  print " -o[N]: Use invariant order selection with threshold N to compute an"
  print "        invariant set; the default value of N is 15"
  print " -q[T]: Use reliability with temperature T to compute gene weights for"
  print "        normalisation function inference; the default value of T is 1.5"
  print " -s[X]: Use reliability with reliabilities measured in terms of the"
  print "        distance to the average reliability multiplied by X divided by"
  print "        the sample standard deviation passed through a sigmoid"
  print "        function; the default value of X is 1"
  print " -p[D]: Use polynomials of degree D as class of normalising functions;"
  print "        this is the default class of functions with the default value"
  print "        of D being 4"
  print " -l[D]: Use loess fits of order D as class of normalising function;"
  print "        the default value of D is 1"
  print " -nA: Set the neighbourhood parameter of loess fits to A; the default"
  print "        value of A is 0.4"
  print " -m: Use robust loess fits as class of normalising functions; to"
  print "     change the order from the default value of 1 use the -l option"
  print " -k[N]: Approximate loess fit with N piecewise polynomials of the same"
  print "        order; the default value of N is the number of measurements"
  print "        divided by 100; this option does not set the type of"
  print "        normalisation function, so it needs to be used in combination"
  print "        with the -l (and possibly the -m) option"
  print " -fN: write output to file named N; this option is useful for sending"
  print "      the normalised data to a file while still reading the"
  print "      unnormalised data from standard input"
  print " -d: Read (and write) the data as a pair of lists; default is as a"
  print "     list of pairs"
  print " --comment=S: Lines starting with the string S are treated as"
  print "              comments. More than one type of comment indicator can"
  print "              be specified."
  print " --skip-initial=N: The first N lines are treated as a header, i.e. as"
  print "                   not containing any measurements. The last value"
  print "                   will be used if this option appears multiple times."
  print " --skip-terminal=N: Similar to --skip-initial, except that it is the"
  print "                    last N lines that are treated as a footer."
  print " --read-at-most=N: Only read N measurements, treating any remaining"
  print "                   part of the input as a footer. If this option"
  print "                   appears multiple times, the smallest value"
  print "                   encountered will be used."
  print " --start-indicator=S: Treat everything as a header until a line"
  print "                      identical to S is encountered. If this option"
  print "                      appears multiple times, lines identical to"
  print "                      each start indicator have to be encountered in"
  print "                      the same order before the header is deemed"
  print "                      finished, and reading of measurements commences."
  print " --end-indicator=S: Similar to --start-indicator, except that the"
  print "                    footer is deemed to commence as soon as any"
  print "                    end indicator specified is encountered."
  print " --column-separator=S: Assume that the columns in a row containing"
  print "                       measurements are separated by the string S. The"
  print "                       default behavious is to assume that columns are"
  print "                       separated by a non-empty string of white spaces"
  print "                       (i.e. spaces and tabs)."
  print " --ignore=R:"
  print " --eliminate=R:"
  print "  Ignore/eliminate measurement rows for which relation R holds."
  print "  Ignored rows are not normalised but copied verbatim to the output."
  print "  Eliminated rows are not normalised and are not copied to the output."
  print "  The relation R should be on the form C#V, where C is the column"
  print "  (numbered from 1) that provides the value from the row, # is one"
  print "  of the comparison operators <, >, =, <=, >=, !=, <<, >>, ==, <<=,"
  print "  >>=, or !!=, and V is the value the row value is compared to. If"
  print "  both the row value and V can be interpreted as a numerical value"
  print "  and the operator is one of < (smaller than), > (larger than),"
  print "  = (equal to), <= (smaller than or equal to), >= (larger than or"
  print "  equal to), or != (not equal to), numerical comparison is used."
  print "  Otherwise textual comparison is used, i.e. the operators <<, >>, ==,"
  print "  <<=, >>=, and !!= forces textual comparison. Mostly textual and"
  print "  numerical comparison yields identical results, but e.g `1<.3' and"
  print "  `1<<.3' will not be the same for rows where the first column is `0.2'."
  print "  More than one relation can be specified."
  print " --column1=N: First sample measurements are taken from column N"
  print "              (numbered from 1). If this option is encountered"
  print "              multiple times, the last column specified is used."
  print "              The default value is 1."
  print " --column2=N: Similar to --column1, but for second sample. The default"
  print "              value is 2 if the second sample is contained in the same"
  print "              file as the first sample, otherwise it is 1."
  print " --group-by=N: The rows are separated into groups according to their"
  print "               value in column N (numbered from 1). Normalisation is"
  print "               done independently for each group. More than one"
  print "               grouping criteria can be specified, in which case the"
  print "               rows have to share the same values in all the columns"
  print "               specified. This option can be used e.g. to do print tip"
  print "               specific normalisation."
  print " --separate-files:"
  print "  Second sample is read from separate file S. If this option is"
  print "  specified, the options --comment1, --comment2, skip-initial1,"
  print "  --skip-initial2, --skip-terminal1, --skip-terminal2, --read-at-most1,"
  print "  --read-at-most2, --start-indicator1, --start-indicator2,"
  print "  --end-indicator1, --end-indicator2, --column-separator1,"
  print "  --column-separator2, --ignore1, --ignore2, eliminate1, --eliminate2,"
  print "  --group-by1, and --group-by2 allow specifying different values of"
  print "  these parameters for the two files. A note of caution: the options"
  print "  not postfixed by 1 or 2 will specify parameters for both files,"
  print "  so e.g. --start-indicator=foo -start-indicator1=bar will effect that"
  print "  we first need to see a line reading `foo' and then a line reading"
  print "  `bar' in the file the first sample is read from before we start"
  print "  parsing lines as containing measurements; in the file the second"
  print "  sample is read from, we need to see a line reading `foo' before"
  print "  we start parsing lines."
  print " --match-by=N --match-by1=N --match-by2=N:"
  print "  If the two sample measurements are read from separate files, the"
  print "  default behaviour is to assume that genes are matched by the order"
  print "  in which they occur in the files. The match-by options allow you"
  print "  to specify one or more columns whose values taken together are"
  print "  used as a key to uniquely identify a gene, e.g. if the first column"
  print "  in both files hold the gene id you should specify this by"
  print "  --match-by=1. A note of warning: if several match-by options are"
  print "  specified, the columns are checked in the order given for each"
  print "  sample, so e.g. --match-by1=1 --match-by=2 --match-by2=3 means that"
  print "  the contents of column 1 for the first sample has to match the"
  print "  contents of column 2 for the second sample, and the contents of"
  print "  column 2 for the first sample has to match the contents of column 3"
  print "  for the second sample. It is an error to specify a different number"
  print "  of match-by columns for the two samples."
  print " --only-keep-matched: Only genes that occurs in both samples and do"
  print "                      not meet an eliminate criterion in either sample"
  print "                      are retained in the output."
  print "In textual arguments to options, the \\ character is treated as an"
  print "escape character - \\t is converted to the tabulator character and"
  print "\\\\ is converted to a single \\ character. If a \\ character is"
  print "followed by anything else, i.e. not a t or \\ character, it is"
  print "removed."
  print "\nExample: Assume that our data file starts with a header where each"
  print "line is prefixed with //, and that each row of measurement consists"
  print "of white space separated columns where the first column is a gene"
  print "identification, the second and third columns are the block row and"
  print "column numbers (i.e. the print tip id), the fourth and fifth columns"
  print "are the first sample flag (with 0 indicating a successful measurement)"
  print "and intensity, and the sixth and seventh columns are the second"
  print "sample flag and intensity. To normalise this data set using the"
  print "invariant order method and loess fits with default parameters and"
  print "print tip specific normalisation, run the program as\n"
  print " " + s + " --comment='//' --group-by=2 --group-by=3 --eliminate='4!=0' --eliminate='6!=0' -o -l <input file> <output file>"
  exit(exit_value)

# Possible relations
def smaller_than(x):
  return x[0] < x[1]
def smaller_than_or_equal(x):
  return x[0] <= x[1]
def equal_to(x):
  return x[0] == x[1]
def not_equal_to(x):
  return x[0] != x[1]
def greater_than(x):
  return x[0] > x[1]
def greater_than_or_equal(x):
  return x[0] >= x[1]
# Check whether the two elements of a pair can be converted to numerical types
def convert2numerical(x):
  # First test whether both elements can be converted to numerical types at all
  try:
    y = (float(x[0]), float(x[1]))
  except:
    return x
  # Now test whether each element can be converted to an integer
  try:
    y = (int(x[0]), y[1])
  except:
    pass
  try:
    y = (y[0], int(x[1]))
  except:
    pass
  return y
# Parse a relation specification
def parse_relation(option):
  def minnonnegative(a):
    # Find the minimum non-negative number in array a
    try:
      return min(filter(lambda x: x >= 0, a))
    except:
      return -1
  # Determine first occurrence of relational symbol
  i = minnonnegative([option.find('<'), option.find('='), option.find('>'), option.find('!')])
  try:
    if i < 0:
      # No relational symbol in option
      raise ValueError
    # Determine column relation is applied to
    column = int(option[:i]) - 1
    if column < 0:
      raise ValueError
    # Determine whether string comparison should be forced
    if option[i + 1] == option[i]:
      operator = []
      j = i + 2
    else:
      operator = [convert2numerical]
      j = i + 1
    # Determine relation
    if option[i] == '<':
      # The relation has something to do with smaller than
      if option[j] == '=':
        # The relation is <=
        operator.append(smaller_than_or_equal)
        j += 1
      else:
        # The relation is <
        operator.append(smaller_than)
    elif option[i] == '>':
      # The relation has something to do with greater than
      if option[j] == '=':
        # The relation is >=
        operator.append(greater_than_or_equal)
        j += 1
      else:
        # The relation is >
        operator.append(greater_than)
    elif option[i] == '!':
      # The option is !=
      operator.append(not_equal_to)
      j += 1
    else:
      # The option is =
      operator.append(equal_to)
    # The value the column has to be in relation to is the remainder of the
    # option. Check whether this can be converted to a numerical value.
    try:
      float(option[j:])
    except:
      # It can't - don't bother trying again
      operator = operator[-1:]
  except:
    stderr.write("ERROR: a relation specification should consist of an integral number, followed\nby a relation (<, >, <=, >=, !=, or =, possibly with the first symbol\nrepeated for forced textual comparison), and finally with the value the\ncolumn is compared to. This is not the case for " + option + "\n")
    raise
  return (column, operator, option[j:])

short_options = "?hHacegb::t::r::i::j:o::q::s::p::l::n:m:k::x:"
long_options = ["comment=", "skip-initial=", "skip-terminal=", "read-at-most=", "start-indicator=", "end-indicator=", "column-separator=", "ignore=", "eliminate=", "column1=", "column2=", "group-by=", "help", "separate-files", "match-by=", "only-keep-matced", "comment1=", "comment2=", "skip-initial1=", "skip-initial2=", "skip-terminal1=", "skip-terminal2=", "read-at-most1=", "read-at-most2=", "start-indicator1=", "start-indicator2=", "end-indicator1=", "end-indicator2=", "column-separator1=", "column-separator2=", "ignore1=", "ignore2=", "eliminate1=", "eliminate2=", "group-by1=", "group-by2=", "match-by1=", "match-by2="]
getopt = MyGetopt(short_options, long_options)
try:
  options, arguments = getopt.parse(argv[1:])
except GetoptError, msg:
  print getopt.long_options
  if msg.msg != None:
    stderr.write(msg.msg)
  print_usage(argv[0])

# Parse options
comment = [[], [], []]
initialskip = [0, 0, 0]
terminalskip = [0, 0, 0]
maxgenes = [None, None, None]
startstring = [[], [], []]
endstring = [[], [], []]
separator = [[], [], []]
ignore = [[], [], []]
eliminate = [[], [], []]
c1 = 0
c2 = None
grouping = [[], [], []]
separate_files = False
matching = [[], []]
retain_singletons = True
amtransform = False
reference = False
logspace = True
exclude = 0
GLOBAL, ITERATED_BISQUARE, ITERATED_GAUSSIAN, INVARIANT_RANK, ITERATED_INVARIANT_RANK, INVARIANT_ORDER, RELIABILITY, SIGMOIDED_RELIABILITY = range(0, 8)
POLYNOMIAL, LOESS, ROBUST_LOESS, APPROXIMATE_LOESS, APPROXIMATE_ROBUST_LOESS = range(0, 5)
invariant_set = GLOBAL
function = POLYNOMIAL
parameter = {}

short_option_letters = reduce(lambda x, y: x + y, getopt.short_options.keys(), "")
def remove_equalsign(s):
  if s:
    if s[-1] == '=':
      if len(s) > 1 and s[-2] == '=':
        return s[:-2]
      return s[:-1]
  return s
long_option_names = map(remove_equalsign, long_options)
def intoption(option):
  # Check whether the option argument is an integer >= 0, and if so return it
  try:
    i = int(option[1])
    if i < 0:
      raise ValueError
    return i
  except:
    stderr.write("ERROR: option " + option[0] + " only accepts a non-negative integer argument, which " + option[1] + " is not\n")
    exit(1)
def positiveintoption(option):
  # Check whether the option argument is an integer > 0, and if so return it
  i = intoption(option)
  if i == 0:
    stderr.write("ERROR: option " + option[0] + " only accepts a positive integer argument, which " + option[1] + " is not\n")
    exit(1)
  return i
def numericaloption(option):
  # Check whether the option argument is a numerical value, and if so return it
  try:
    i = float(option[1])
    return i
  except:
    stderr.write("ERROR: option " + option[0] + " only accepts a numerical argument, which " + option[1] + " is not\n")
    exit(1)
def positiveoption(option):
  # Check whether the option argument is an integer > 0, and if so return it
  i = floatoption(option)
  if i == 0:
    stderr.write("ERROR: option " + option[0] + " only accepts a positive numerical argument, which " + option[1] + " is not\n")
    exit(1)
  return i

def convert_escapees(s):
  # Convert escape sequences
  i = 0
  while True:
    j = s[i:].find("\\")
    if j < 0:
      # No more escaped characters
      break
    i += j
    if i == len(s) - 1:
      # Terminal backslash
      break
    elif s[i + 1] == 't':
      # Tab character escape sequence
      s = s[:i] + '\t' + s[i + 2:]
      i += 1
    else:
      # Non special escape sequence - remove backslash
      s = s[:i] + s[i + 1:]
      i += 1
  return s

for option in options:
  if option[0][1] != '-':
    # Short option
    if option[0][1] in "?hH":
      # Help requested
      print_usage(argv[0])
    elif option[0][1] == 'a':
      amtransform = True
    elif option[0][1] == 'c':
      reference = True
    elif option[0][1] == 'e':
      logspace = False
    elif option[0][1] == 'g':
      invariant_set = GLOBAL
    elif option[0][1] == 'b':
      invariant_set = ITERATED_BISQUARE
      if option[1] != None:
        parameter['b'] = positiveoption(option)
    elif option[0][1] == 't':
      invariant_set = ITERATED_GAUSSIAN
      if option[1] != None:
        parameter['t'] = positiveoption(option)
    elif option[0][1] == 'r':
      invariant_set = INVARIANT_RANK
      if option[1] != None:
        parameter['r'] = intoption(option)
    elif option[0][1] == 'i':
      invariant_set = ITERATED_INVARIANT_RANK
      if option[1] != None:
        parameter['i'] = positiveoption(option)
    elif option[0][1] == 'j':
      parameter['j'] = positiveoption(option)
    elif option[0][1] == 'o':
      invariant_set = INVARIANT_ORDER
      if option[1] != None:
        parameter['o'] = intoption(option)
    elif option[0][1] == 'q':
      invariant_set = RELIABILITY
      if option[1] != None:
        parameter['q'] = positiveoption(option)
    elif option[0][1] == 's':
      invariant_set = SIGMOIDED_RELIABILITY
      if option[1] != None:
        parameter['s'] = positiveoption(option)
    elif option[0][1] == 'p':
      function = POLYNOMIAL
      if option[1] != None:
        parameter['p'] = intoption(option)
    elif option[0][1] == 'l':
      function = LOESS
      if option[1] != None:
        parameter['l'] = positiveintoption(option)
    elif option[0][1] == 'n':
      parameter['n'] = positiveoption(option)
    elif option[0][1] == 'm':
      function = ROBUST_LOESS
      # As we have encountered the m option, loess fits should be robust
      LOESS = ROBUST_LOESS
      APPROXIMATE_LOESS = APPROXIMATE_ROBUST_LOESS
    elif option[0][1] == 'k':
      # As we have encountered the k option, loess fits should be approximate
      LOESS = APPROXIMATE_LOESS
      ROBUST_LOESS = APPROXIMATE_ROBUST_LOESS
      if option[1] != None:
        parameter['k'] = positiveintoption(option)
    elif option[0][1] == 'x':
      exclude = intoption(option)
  else:
    # Long option
    if option[0][2:] == "comment":
      # Prefix defining comment line
      comment[0].append(convert_escapees(option[1]))
    if option[0][2:] == "comment1":
      # Prefix defining comment line
      comment[1].append(convert_escapees(option[1]))
    if option[0][2:] == "comment2":
      # Prefix defining comment line
      comment[2].append(convert_escapees(option[1]))
    elif option[0][2:] == "skip-initial":
      # Number of lines to skip initially
      initialskip = 3 * [intoption(option)]
    elif option[0][2:] == "skip-initial1":
      # Number of lines to skip initially
      initialskip[1] = intoption(option)
    elif option[0][2:] == "skip-initial2":
      # Number of lines to skip initially
      initialskip[2] = intoption(option)
    elif option[0][2:] == "skip-terminal":
      # Number of lines to skip at end of input
      terminalskip = 3 * [intoption(option)]
    elif option[0][2:] == "skip-terminal1":
      # Number of lines to skip at end of input
      terminalskip[1] = intoption(option)
    elif option[0][2:] == "skip-terminal2":
      # Number of lines to skip at end of input
      terminalskip[2] = intoption(option)
    elif option[0][2:] == "read-at-most":
      # Maximum number of genes to read - keep minimum number encountered
      i = positiveintoption(option)
      if maxgenes[0]:
        maxgenes[0] = min(maxgenes[0], i)
        maxgenes[1] = min(maxgenes[1], i)
        maxgenes[2] = min(maxgenes[2], i)
      else:
        maxgenes[0] = i
        if maxgenes[1]:
          maxgenes[1] = min(maxgenes[1], i)
        else:
          maxgenes[1] = i
        if maxgenes[2]:
          maxgenes[2] = min(maxgenes[2], i)
        else:
          maxgenes[2] = i
    elif option[0][2:] == "read-at-most1":
      # Maximum number of genes to read - keep minimum number encountered
      if maxgenes[1]:
        maxgenes[1] = min(maxgenes[1], positiveintoption(option))
      else:
        maxgenes[1] = positiveintoption(option)
    elif option[0][2:] == "read-at-most2":
      # Maximum number of genes to read - keep minimum number encountered
      if maxgenes[2]:
        maxgenes[2] = min(maxgenes[2], positiveintoption(option))
      else:
        maxgenes[2] = positiveintoption(option)
    elif option[0][2:] == "start-indicator":
      # Special line indicating end of header
      s = convert_escapees(option[1])
      startstring[0].append(s)
      startstring[1].append(s)
      startstring[2].append(s)
    elif option[0][2:] == "start-indicator1":
      # Special line indicating end of header
      startstring[1].append(convert_escapees(option[1]))
    elif option[0][2:] == "start-indicator2":
      # Special line indicating end of header
      startstring[2].append(convert_escapees(option[1]))
    elif option[0][2:] == "end-indicator":
      # Special line indicating start of footer
      s = convert_escapees(option[1])
      endstring[0].append(s)
      endstring[1].append(s)
      endstring[2].append(s)
    elif option[0][2:] == "end-indicator1":
      # Special line indicating start of footer
      endstring[1].append(convert_escapees(option[1]))
    elif option[0][2:] == "end-indicator2":
      # Special line indicating start of footer
      endstring[2].append(convert_escapees(option[1]))
    elif option[0][2:] == "column-separator":
      # String used to separate columns
      s = convert_escapees(option[1])
      separator[0].append(s)
      separator[1].append(s)
      separator[2].append(s)
    elif option[0][2:] == "column-separator1":
      # String used to separate columns
      separator[1].append(convert_escapees(option[1]))
    elif option[0][2:] == "column-separator2":
      # String used to separate columns
      separator[2].append(convert_escapees(option[1]))
    elif option[0][2:] == "ignore":
      # A relation specifying rows that should be ignored, i.e. not normalised
      # but still retained in the output
      try:
        r = parse_relation(convert_escapees(option[1]))
        ignore[0].append(r)
        ignore[1].append(r)
        ignore[2].append(r)
      except:
        # Something went wrong
        print_usage(argv[0])
    elif option[0][2:] == "ignore1":
      # A relation specifying rows that should be ignored, i.e. not normalised
      # but still retained in the output
      try:
        ignore[1].append(parse_relation(convert_escapees(option[1])))
      except:
        # Something went wrong
        print_usage(argv[0])
    elif option[0][2:] == "ignore2":
      # A relation specifying rows that should be ignored, i.e. not normalised
      # but still retained in the output
      try:
        ignore[2].append(parse_relation(convert_escapees(option[1])))
      except:
        # Something went wrong
        print_usage(argv[0])
    elif option[0][2:] == "eliminate":
      # A relation specifying rows that should be eliminated, i.e. also removed
      # from the output
      try:
        r = parse_relation(convert_escapees(option[1]))
        eliminate[0].append(r)
        eliminate[1].append(r)
        eliminate[2].append(r)
      except:
        # Something went wrong
        print_usage(argv[0])
    elif option[0][2:] == "eliminate1":
      # A relation specifying rows that should be eliminated, i.e. also removed
      # from the output
      try:
        eliminate[1].append(parse_relation(convert_escapees(option[1])))
      except:
        # Something went wrong
        print_usage(argv[0])
    elif option[0][2:] == "eliminate2":
      # A relation specifying rows that should be eliminated, i.e. also removed
      # from the output
      try:
        eliminate[2].append(parse_relation(convert_escapees(option[1])))
      except:
        # Something went wrong
        print_usage(argv[0])
    elif option[0][2:] == "column1":
      # Index of first data column
      c1 = positiveintoption(option) - 1
    elif option[0][2:] == "column2":
      # Index of second data column
      c2 = positiveintoption(option) - 1
    elif option[0][2:] == "group-by":
      # Specification of column data is groupd by
      i = positiveintoption(option) - 1
      grouping[0].append(i)
      grouping[1].append(i)
      grouping[2].append(i)
    elif option[0][2:] == "group-by1":
      # Specification of column data is groupd by
      grouping[1].append(positiveintoption(option) - 1)
    elif option[0][2:] == "group-by2":
      # Specification of column data is groupd by
      grouping[2].append(positiveintoption(option) - 1)
    elif option[0][2:] == "second-file":
      separate_files = True
    elif option[0][2:] == "separate-files":
      # Read samples from separate files
      separate_files = True
    elif option[0][2:] == "match-by":
      # Specification of column data is groupd by
      i = positiveintoption(option) - 1
      matching[0].append(i)
      matching[1].append(i)
    elif option[0][2:] == "match-by1":
      # Specification of column data is groupd by
      matching[0].append(positiveintoption(option) - 1)
    elif option[0][2:] == "match-by2":
      # Specification of column data is groupd by
      matching[1].append(positiveintoption(option) - 1)
    elif option[0][2:] == "only-keep-matched":
      # Throw away genes that do not successfully occur in both samples
      retain_singletons = False
    elif option[0][2:] == "help":
      # Help requested
      print_usage(argv[0])

def istvan_options(invariant_set, function, parameter, amtransform, reference, logspace, exclude):
  # Generate string representation of options governing normalisation method
  # Handle part specifying the invariant set method
  if invariant_set == ITERATED_BISQUARE:
    if parameter.has_key('b'):
      options = " -b" + `parameter['b']`
    else:
      options = " -b6"
  elif invariant_set == ITERATED_GAUSSIAN:
    if parameter.has_key('t'):
      options = " -t" + `parameter['t']`
    else:
      options = " -t1"
  elif invariant_set == INVARIANT_RANK:
    if parameter.has_key('r'):
      options = " -r" + `parameter['r']`
    else:
      options = " -r15"
  elif invariant_set == ITERATED_INVARIANT_RANK:
    if parameter.has_key('i'):
      options = " -i" + `parameter['i']`
    else:
      options = " -i0.02"
    if parameter.has_key('j'):
      options += " -j" + `parameter['j']`
  elif invariant_set == INVARIANT_ORDER:
    if parameter.has_key('o'):
      options = " -o" + `parameter['o']`
    else:
      options = " -o15"
  elif invariant_set == RELIABILITY:
    if parameter.has_key('q'):
      options = " -q" + `parameter['q']`
    else:
      options = " -q1.5"
  elif invariant_set == SIGMOIDED_RELIABILITY:
    if parameter.has_key('s'):
      options = " -s" + `parameter['s']`
    else:
      options = " -s1"
  else:
    options = " -g"
  # Handle part specifying the type of function inferred
  if function == POLYNOMIAL:
    # Use polynomials
    if parameter.has_key('p'):
      options += " -p" + `parameter['p']`
    else:
      options += " -p4"
  else:
    # Use loess fits
    if parameter.has_key('l'):
      options += " -l" + `parameter['l']`
    else:
      options += " -l1"
    if function == ROBUST_LOESS or function == APPROXIMATE_ROBUST_LOESS:
      options += " -m"
    if function == APPROXIMATE_LOESS or function == APPROXIMATE_ROBUST_LOESS:
      if parameter.has_key('k'):
        options += " -k" + `parameter['k']`
      else:
        options += " -k"
    if parameter.has_key('n'):
      options += " -n" + `parameter['n']`
  if amtransform == True:
    options += " -a"
    if reference == True:
      options += " -c"
  if logspace == False:
    options += " -e"
  if exclude > 0:
    options += " -x" + `exclude`
  return options
# Finalise options
if c2 == None:
  if separate_files:
    c2 = 0
  else:
    c2 = 1

# Parse arguments
if arguments:
  # Prepare files for reading and writing sample data
  try:
    infile = open(arguments[0], 'r')
  except:
    stderr.write("ERROR: could not open file " + arguments[0] + " for input\n")
    exit(1)
  if arguments[1:]:
    try:
      outfile = open(arguments[1], 'w')
    except:
      stderr.write("ERROR: could not open file " + arguments[1] + " for output\n")
      exit(1)
  else:
    outfile = stdout
  if separate_files:
    # Prepare files for reading and writing second sample data
    arguments = arguments[2:]
    if arguments:
      try:
        infile2 = open(arguments[0], 'r')
      except:
        stderr.write("ERROR: could not open file " + arguments[0] + " for input\n")
        exit(1)
      if arguments[1:]:
        try:
          outfile2 = open(arguments[1], 'w')
        except:
          stderr.write("ERROR: could not open file " + arguments[1] + " for output\n")
          exit(1)
      else:
        outfile2 = stdout
  if arguments[2:]:
    # Too many arguments
    stderr.write("WARNING: unused argument")
    if arguments[3:]:
      stderr.write("s:\n")
    else:
      stderr.write(":\n")
    for argument in arguments[2:]:
      stderr.write(" " + argument + "\n")
else:
  infile, outfile = stdin, stdout
  if separate_files:
    infile2, outfile2 = stdin, stdout

# Read input data
def read_data(infile, initialskip, terminalskip, maxgenes, startstring, endstring, comment, separator):
  input = infile.readlines()
  data = []
  # Use i to keep track of line number and j to keep track of gene index
  i, j = initialskip - 1, 0
  for line in input[initialskip:len(input) - terminalskip]:
    i += 1
    # Check whether we still need to see a special start string
    if startstring:
      if line == startstring[0]:
        # Encountered (next) end of header string
        startstring = startstring[1:]
      continue
    # Check whether we have reached a special end string
    def isendstring(line, endstrings):
      for endstring in endstrings:
        if line == endstring:
          # Reached the end of the data
          return 1
      return 0
    if isendstring(line, endstring):
      break
    # Check whether the line is a comment
    def iscomment(line, comments):
      for comment in comments:
        if comment == line[:len(comment)]:
          return 1
      return 0
    if iscomment(line, comment):
      continue

    # We have identified a genuine line of data
    data.append(mysplit(line, separator))
    data[-1] = [i] + data[-1]
    j += 1
    # Check whether we have now read a specified number of genes
    if maxgenes and j == maxgenes:
      break
  return input, data

def output_data(data, input, eliminate, outfile):
  # Make sure we read the data in the right order
  eliminate.sort()
  data.sort()
  # Output normalised data
  j = 0
  k = 0
  for i in range(0, len(input)):
    try:
      if i == eliminate[j]:
        # This line met the eliminate criterion
        j += 1
        continue
    except IndexError:
      pass
    try:
      if i == data[k][0]:
        # Next line is a measurement line
        outfile.write(myjoin(data[k][1:]))
        k += 1
        continue
    except IndexError:
      # If we are out of measurement lines we may as well just copy the input
      if j >= len(eliminate):
        for j in range(i, len(input)):
          outfile.write(input[j])
        break
    # Next line did not contain measurement data
    outfile.write(input[i])
  if outfile != stdout:
    outfile.close()

if separate_files:
  # Read samples from separate files
  input1, data1 = read_data(infile, initialskip[1], terminalskip[1], maxgenes[1], startstring[1], endstring[1], comment[1], separator[1])
  if infile != stdin:
    infile.close()
  input2, data2 = read_data(infile2, initialskip[2], terminalskip[2], maxgenes[2], startstring[2], endstring[2], comment[2], separator[2])
  if infile2 != stdin:
    infile2.close()
  # Do the normalisation
  data, eliminated = match_genes((data1, data2), matching, eliminate[1:], ignore[1:], retain_singletons)
  normalise_twofiles(data, (c1, c2), grouping[1:], istvan_options(invariant_set, function, parameter, amtransform, reference, logspace, exclude))
  # Output normalised data
  output_data(map(lambda x: x[0], data), input1, eliminated[0], outfile)
  output_data(map(lambda x: x[1], data), input2, eliminated[1], outfile2)
else:
  # Read samples from one file
  input, data = read_data(infile, initialskip[0], terminalskip[0], maxgenes[0], startstring[0], endstring[0], comment[0], separator[0])
  if infile != stdin:
    infile.close()
  # Remove rows that obey at least one of the eliminate relations
  data, eliminate = check_relations(data, eliminate[0])
  eliminate = map(lambda x: x[0], eliminate)
  # Do the normalisation
  normalise_onefile(data, c1, c2, ignore[0], grouping[0], istvan_options(invariant_set, function, parameter, amtransform, reference, logspace, exclude))
  # Output normalised data
  output_data(data, input, eliminate, outfile)
