/*******************************************************************

    llist.h
  
    Description of a general purpose linked list with a stack and
    queue interface etc.
		
    Christian Storm (cstorm@daimi.aau.dk), May 1995

********************************************************************/

#ifndef _LLIST_H
#define _LLIST_H

#include <stdarg.h>

#define FIRST -1
#define LAST  -2

/* Datatypes */

typedef struct tagLListNode
{
  void *elm;			
  struct tagLListNode *prev;
  struct tagLListNode *next;
} LListNode;

typedef struct tagLList
{
  int count;
  struct tagLListNode *first;
  struct tagLListNode *last;
} LList;

typedef struct tagLListCounter
{
  int pos;
  struct tagLListNode *current;
} LListCounter;

/* Prototypes */

LList *MakeLList();
LList *MakeLLists(int n);
void PreallocateLListNodes(int n);
void FreeUnusedLListNodes();
void DestroyLList(LList *llist, void (*destroy_elm)(void *));
void DestroyLLists(LList *llist, int n, void (*destroy_elm)(void *));
void DeleteLList(LList *llist, void (*delete_elm)(void *));
void DeleteLLists(LList *llist, int n, void (*delete_elm)(void *));
void Insert(LList *llist, LListNode *lnode, void *elm);
void *Remove(LList *llist, LListNode *lnode);
void Push(LList *llist, void *elm);
void *Pop(LList *llist);
void *Top(LList *llist);
void Enqueue(LList *llist, void *elm);
void *Dequeue(LList *llist);
void *Bottom(LList *llist);
void LListMap(LList *llist, void (*f)(void *, va_list), ...);
LListCounter *MakeCounter(LList *llist, int pos);
void InitCounter(LListCounter *lcounter, LList *llist, int pos);
void *SetCounter(LListCounter *lcounter, int pos);
void *Next(LListCounter *lcounter);
void *Prev(LListCounter *lcounter);
void *RemoveMoveLeft(LList *llist, LListCounter *lcounter);
void *RemoveMoveRight(LList *llist, LListCounter *lcounter);
int Length(LList *llist);

#endif
