/* lsd_polynomial.h: Function prototype and global definitions for
 * various functions on polynomials, most importantly computing the
 * polynomial minimising the sum of square differences to a weighted
 * set of points. These functions are only intended for use with low
 * order polynomials, with higher order polynomials the methods should
 * be updated with a more stable method for finding the least square
 * differences polynomial and FFT for multiplying two polynomials.
 */
/* Rune Lyngs, January 2003 */
#ifndef _POLYNOMIAL_H
#define _POLYNOMIAL_H

/* _LSD_POLYNOMIAL_THRESHOLD determines maximal point-wise deviance
 * from identity matrix for A * A^{-1} before an instability warning
 * is output to stderr, where A^{-1} is the inverse matrix computed
 * from the equation system matrix A.
 */
#define _POLYNOMIAL_THRESHOLD 10000.0001

typedef struct {
  int n;     /* Number of coefficients for polynomial, i.e. degree of
	      * polynomial plus one. */
  double *a; /* Coefficients of polynomial */
} Polynomial;

/* Descriptions of functions precede the implementation of the
 * function in polynomial.c.
 */
Polynomial *polynomial_new(unsigned int n, double *a);
Polynomial *polynomial_wlsdpolynomial(int n, int m, double *x, double *y,
				     double *w, double p, double a, double b);
Polynomial *polynomial_lsdpolynomial(int n, int m, double *x, double *y,
				     double p, double a, double b);
Polynomial *polynomial_fit(int n, double *x, double *y);
Polynomial *polynomial_product(Polynomial *p1, Polynomial *p2);
Polynomial *polynomial_difference(Polynomial *p1, Polynomial *p2);
double polynomial_integral(Polynomial *p, double a, double b);
double polynomial_evaluate(Polynomial *p, double x);
double polynomial_isd(Polynomial *p1, Polynomial *p2, double a, double b);
void polynomial_print(Polynomial *p, char *format, FILE *fp);
void polynomial_free(Polynomial *p);

#endif
